@@ -1,6 +1,6 @@
 use std::fs::File;
-use web_audio_api::context::{AsBaseAudioContext, AudioContext};
-use web_audio_api::media::{MediaElement, OggVorbisDecoder};
+use web_audio_api::context::{AudioContext, BaseAudioContext};
+use web_audio_api::media::{MediaDecoder, MediaElement};
 use web_audio_api::node::{AudioControllableSourceNode, AudioNode, AudioScheduledSourceNode};
 
 fn main() {
@@ -8,9 +8,9 @@ fn main() {
 
     // setup background music:
     // read from local file
-    let file = File::open("sample.ogg").unwrap();
+    let file = File::open("samples/major-scale.ogg").unwrap();
     // decode file to media stream
-    let stream = OggVorbisDecoder::try_new(file).unwrap();
+    let stream = MediaDecoder::try_new(file).unwrap();
     // wrap stream in MediaElement, so we can control it (loop, play/pause)
     let media = MediaElement::new(stream);
     // register as media element in the audio context
