@@ -1,7 +1,7 @@
 use rand::rngs::ThreadRng;
 use rand::Rng;
 use std::{thread, time};
-use web_audio_api::context::{AsBaseAudioContext, AudioContext};
+use web_audio_api::context::{AudioContext, BaseAudioContext};
 use web_audio_api::node::{AudioNode, AudioScheduledSourceNode};
 
 // run in release mode
@@ -9,7 +9,7 @@ use web_audio_api::node::{AudioNode, AudioScheduledSourceNode};
 
 fn trigger_sine(audio_context: &AudioContext, rng: &mut ThreadRng) {
     let env = audio_context.create_gain();
-    env.gain().set_value(0.05);
+    env.gain().set_value(0.);
     env.connect(&audio_context.destination());
 
     let osc = audio_context.create_oscillator();
@@ -20,9 +20,9 @@ fn trigger_sine(audio_context: &AudioContext, rng: &mut ThreadRng) {
     let freq = rng.gen_range(100..3000) as f32;
     osc.frequency().set_value(freq);
 
-    env.gain().set_value_at_time(0., now);
-    env.gain().linear_ramp_to_value_at_time(0.1, now + 0.01);
     env.gain()
+        .set_value_at_time(0., now)
+        .linear_ramp_to_value_at_time(0.1, now + 0.01)
         .exponential_ramp_to_value_at_time(0.0001, now + 2.);
 
     osc.start_at(now);
