@@ -1,11 +1,13 @@
-use web_audio_api::context::{AsBaseAudioContext, AudioContext, AudioContextOptions, LatencyHint};
+use web_audio_api::context::{
+    AudioContext, AudioContextLatencyCategory, AudioContextOptions, BaseAudioContext,
+};
 use web_audio_api::node::{AudioNode, AudioScheduledSourceNode};
 
 fn main() {
     let options = AudioContextOptions {
         sample_rate: Some(48_000),
         channels: Some(2),
-        latency_hint: Some(LatencyHint::Playback),
+        latency_hint: Some(AudioContextLatencyCategory::Playback),
     };
 
     let context = AudioContext::new(Some(options));
@@ -25,9 +27,9 @@ fn main() {
     let merger = context.create_channel_merger(2);
 
     // connect left osc to left input of the merger
-    left.connect_at(&merger, 0, 0).unwrap();
+    left.connect_at(&merger, 0, 0);
     // connect right osc to left input of the merger
-    right.connect_at(&merger, 0, 1).unwrap();
+    right.connect_at(&merger, 0, 1);
 
     // Connect the merger to speakers
     merger.connect(&context.destination());
