@@ -6,11 +6,14 @@ use crate::render::AudioRenderQuantum;
 use crate::SampleRate;
 
 /// Options for constructing an [`AudioBuffer`]
-#[derive(Copy, Clone, Debug)]
+#[derive(Clone, Debug)]
 pub struct AudioBufferOptions {
-    pub number_of_channels: usize, // defaults to 1
-    pub length: usize,             // required
-    pub sample_rate: SampleRate,   // required
+    /// The number of channels for the buffer
+    pub number_of_channels: usize,
+    /// The length in sample frames of the buffer
+    pub length: usize,
+    /// The sample rate in Hz for the buffer
+    pub sample_rate: SampleRate,
 }
 
 /// Memory-resident audio asset, basically a matrix of channels * samples
@@ -19,7 +22,7 @@ pub struct AudioBufferOptions {
 ///
 /// - MDN documentation: <https://developer.mozilla.org/en-US/docs/Web/API/AudioBuffer>
 /// - specification: <https://webaudio.github.io/web-audio-api/#AudioBuffer>
-/// - see also: [`AsBaseAudioContext::create_buffer`](crate::context::AsBaseAudioContext::create_buffer)
+/// - see also: [`BaseAudioContext::create_buffer`](crate::context::BaseAudioContext::create_buffer)
 #[derive(Clone, Debug)]
 pub struct AudioBuffer {
     channels: Vec<ChannelData>,
@@ -30,7 +33,13 @@ use std::error::Error;
 
 impl AudioBuffer {
     /// Allocate a silent audiobuffer with [`AudioBufferOptions`]
+    ///
+    /// # Panics
+    ///
+    /// This function will panic if the channel count is zero.
     pub fn new(options: AudioBufferOptions) -> Self {
+        assert!(options.number_of_channels > 0);
+
         let silence = ChannelData::new(options.length);
 
         Self {
@@ -44,9 +53,12 @@ impl AudioBuffer {
     /// The outer Vec determine the channels. The inner Vecs should have the same length.
     ///
     /// # Panics
-    /// This function will panic if `samples` is an empty Vec or if any of its elements have
-    /// different lengths.
+    ///
+    /// This function will panic if `samples` is an empty Vec or any of its items have different
+    /// lengths.
     pub fn from(samples: Vec<Vec<f32>>, sample_rate: SampleRate) -> Self {
+        assert!(!samples.is_empty());
+
         let channels: Vec<_> = samples.into_iter().map(ChannelData::from).collect();
         if !channels.iter().all(|c| c.len() == channels[0].len()) {
             panic!("Trying to create AudioBuffer from channel data with unequal length");
@@ -68,7 +80,13 @@ impl AudioBuffer {
     }
 
     /// Sample rate of this `AudioBuffer` in Hertz
-    pub fn sample_rate(&self) -> SampleRate {
+    pub fn sample_rate(&self) -> f32 {
+        self.sample_rate.0 as f32
+    }
+
+    /// The raw sample rate of the `AudioBuffer` (which has more precision than the float
+    /// [`sample_rate()`](AudioBuffer::sample_rate) value).
+    pub fn sample_rate_raw(&self) -> SampleRate {
         self.sample_rate
     }
 
@@ -203,7 +221,7 @@ impl AudioBuffer {
 
     /// Split an AudioBuffer in two at the given index.
     pub(crate) fn split_off(&mut self, index: usize) -> Self {
-        let sample_rate = self.sample_rate();
+        let sample_rate = self.sample_rate_raw();
 
         let channels: Vec<_> = self
             .channels_mut()
@@ -237,7 +255,7 @@ impl AudioBuffer {
     // assert_eq!(buffer.sample_rate().0, 96_000);
     // ```
     pub(crate) fn resample(&mut self, sample_rate: SampleRate) {
-        if self.sample_rate() == sample_rate {
+        if self.sample_rate_raw() == sample_rate {
             return;
         }
 
@@ -361,7 +379,7 @@ impl<M: MediaStream> Resampler<M> {
 }
 
 impl<M: MediaStream> Iterator for Resampler<M> {
-    type Item = Result<AudioBuffer, Box<dyn Error + Send>>;
+    type Item = Result<AudioBuffer, Box<dyn Error + Send + Sync>>;
 
     fn next(&mut self) -> Option<Self::Item> {
         let mut buffer = match self.buffer.take() {
@@ -428,10 +446,32 @@ mod tests {
 
         assert_eq!(audio_buffer.number_of_channels(), 1);
         assert_eq!(audio_buffer.length(), 10);
-        assert_eq!(audio_buffer.sample_rate().0, 1);
+        assert_float_eq!(audio_buffer.sample_rate(), 1., abs <= 0.);
+        assert_eq!(audio_buffer.sample_rate_raw().0, 1);
         assert_float_eq!(audio_buffer.duration(), 10., abs <= 0.);
     }
 
+    #[test]
+    #[should_panic]
+    fn test_zero_channels() {
+        let options = AudioBufferOptions {
+            number_of_channels: 0,
+            length: 10,
+            sample_rate: SampleRate(1),
+        };
+
+        AudioBuffer::new(options); // should panic
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_zero_channels_from() {
+        let samples = vec![];
+        let sample_rate = SampleRate(1);
+
+        AudioBuffer::from(samples, sample_rate); // should panic
+    }
+
     #[test]
     fn test_copy_from_channel() {
         let options = AudioBufferOptions {
@@ -487,7 +527,7 @@ mod tests {
 
         {
             // same size
-            let mut audio_buffer = AudioBuffer::new(options);
+            let mut audio_buffer = AudioBuffer::new(options.clone());
             let src = vec![1.; 10];
             audio_buffer.copy_to_channel(&src, 0);
             assert_float_eq!(
@@ -499,7 +539,7 @@ mod tests {
 
         {
             // smaller source
-            let mut audio_buffer = AudioBuffer::new(options);
+            let mut audio_buffer = AudioBuffer::new(options.clone());
             let src = vec![1.; 5];
             audio_buffer.copy_to_channel(&src, 0);
             assert_float_eq!(
@@ -511,7 +551,7 @@ mod tests {
 
         {
             // larger source
-            let mut audio_buffer = AudioBuffer::new(options);
+            let mut audio_buffer = AudioBuffer::new(options.clone());
             let src = vec![1.; 12];
             audio_buffer.copy_to_channel(&src, 0);
             assert_float_eq!(
@@ -523,7 +563,7 @@ mod tests {
 
         {
             // w/ offset
-            let mut audio_buffer = AudioBuffer::new(options);
+            let mut audio_buffer = AudioBuffer::new(options.clone());
             let src = vec![1.; 10];
             audio_buffer.copy_to_channel_with_offset(&src, 0, 5);
             assert_float_eq!(
@@ -558,7 +598,7 @@ mod tests {
 
         assert_eq!(b.length(), 10);
         assert_eq!(b.number_of_channels(), 2);
-        assert_eq!(b.sample_rate().0, 44_100);
+        assert_eq!(b.sample_rate_raw().0, 44_100);
         assert_float_eq!(b.channel_data(0).as_slice(), &[0.; 10][..], abs_all <= 0.);
         assert_float_eq!(b.channel_data(1).as_slice(), &[0.; 10][..], abs_all <= 0.);
         assert_eq!(b.channels().get(2), None);
@@ -571,13 +611,13 @@ mod tests {
             length: 5,
             sample_rate: SampleRate(44_100),
         };
-        let mut b1 = AudioBuffer::new(options);
+        let mut b1 = AudioBuffer::new(options.clone());
         let b2 = AudioBuffer::new(options);
         b1.extend(&b2);
 
         assert_eq!(b1.length(), 10);
         assert_eq!(b1.number_of_channels(), 2);
-        assert_eq!(b1.sample_rate().0, 44_100);
+        assert_eq!(b1.sample_rate_raw().0, 44_100);
 
         let channel_data = ChannelData::from(vec![1.; 5]);
         let b3 = AudioBuffer::from_channels(vec![channel_data; 2], SampleRate(44_100));
@@ -586,7 +626,7 @@ mod tests {
 
         assert_eq!(b1.length(), 15);
         assert_eq!(b1.number_of_channels(), 2);
-        assert_eq!(b1.sample_rate().0, 44_100);
+        assert_eq!(b1.sample_rate_raw().0, 44_100);
         assert_float_eq!(
             b1.channel_data(0).as_slice(),
             &[0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 1., 1., 1., 1.][..],
@@ -604,7 +644,7 @@ mod tests {
             &[1., 1., 2., 2., 3., 3., 4., 4., 5., 5.,][..],
             abs_all <= 0.
         );
-        assert_eq!(buffer.sample_rate().0, 200);
+        assert_eq!(buffer.sample_rate_raw().0, 200);
     }
 
     #[test]
@@ -617,7 +657,7 @@ mod tests {
             &[2., 4.][..],
             abs_all <= 0.
         );
-        assert_eq!(buffer.sample_rate().0, 100);
+        assert_eq!(buffer.sample_rate_raw().0, 100);
     }
 
     #[test]
