@@ -19,16 +19,12 @@ const LISTENER_NODE_ID: u64 = 1;
 /// listener audio parameters ids are always at index 2 through 12
 const LISTENER_PARAM_IDS: Range<u64> = 2..12;
 
-use crate::buffer::{AudioBuffer, AudioBufferOptions, ChannelData};
-use crate::media::{MediaElement, MediaStream};
+use crate::buffer::{AudioBuffer, AudioBufferOptions};
+use crate::media::{MediaDecoder, MediaElement, MediaStream};
 use crate::message::ControlMessage;
-use crate::node::{
-    self, AnalyserOptions, AudioBufferSourceOptions, AudioNode, ChannelConfigOptions,
-    ChannelCountMode, ChannelInterpretation, ChannelMergerOptions, ChannelSplitterOptions,
-    ConstantSourceOptions, DelayOptions, GainOptions, IirFilterOptions, PannerOptions,
-    PeriodicWave, PeriodicWaveOptions,
-};
-use crate::param::{AudioParam, AudioParamEvent, AudioParamOptions};
+use crate::node::{self, AudioNode, ChannelConfigOptions};
+use crate::param::{AudioParam, AudioParamDescriptor, AudioParamEvent};
+use crate::periodic_wave::{PeriodicWave, PeriodicWaveOptions};
 use crate::render::{AudioProcessor, NodeIndex, RenderThread};
 use crate::spatial::{AudioListener, AudioListenerParams};
 use crate::{SampleRate, RENDER_QUANTUM_SIZE};
@@ -41,25 +37,33 @@ use cpal::{traits::StreamTrait, Stream};
 
 use crossbeam_channel::Sender;
 
-/// The `BaseAudioContext` interface represents an audio-processing graph built from audio modules
-/// linked together, each represented by an `AudioNode`. An audio context controls both the creation
-/// of the nodes it contains and the execution of the audio processing, or decoding.
+/// The struct that corresponds to the Javascript `BaseAudioContext` object.
+///
+/// Please note that in rust, we need to differentiate between the [`BaseAudioContext`] trait and
+/// the [`ConcreteBaseAudioContext`] concrete implementation.
+///
+/// This object is returned from the `base()` method on [`AudioContext`] and
+/// [`OfflineAudioContext`], or the `context()` method on `AudioNode`s.
+///
+/// The `ConcreteBaseAudioContext` allows for cheap cloning (using an `Arc` internally).
 // the naming comes from the web audio specfication
 #[allow(clippy::module_name_repetitions)]
 #[derive(Clone)]
-pub struct BaseAudioContext {
-    /// inner makes `BaseAudioContextInner` cheap to clone
-    inner: Arc<BaseAudioContextInner>,
+pub struct ConcreteBaseAudioContext {
+    /// inner makes `ConcreteBaseAudioContext` cheap to clone
+    inner: Arc<ConcreteBaseAudioContextInner>,
 }
 
-impl PartialEq for BaseAudioContext {
+impl PartialEq for ConcreteBaseAudioContext {
     fn eq(&self, other: &Self) -> bool {
         Arc::ptr_eq(&self.inner, &other.inner)
     }
 }
 
-/// Inner representation of the `BaseAudioContext`
-struct BaseAudioContextInner {
+/// Inner representation of the `ConcreteBaseAudioContext`
+///
+/// These fields are wrapped inside an `Arc` in the actual `ConcreteBaseAudioContext`.
+struct ConcreteBaseAudioContextInner {
     /// sample rate in Hertz
     sample_rate: SampleRate,
     /// number of speaker output channels
@@ -76,88 +80,71 @@ struct BaseAudioContextInner {
     listener_params: Option<AudioListenerParams>,
 }
 
-/// Retrieve the `BaseAudioContext` from the concrete `AudioContext`
+/// The interface representing an audio-processing graph built from audio modules linked together,
+/// each represented by an `AudioNode`.
+///
+/// An audio context controls both the creation of the nodes it contains and the execution of the
+/// audio processing, or decoding.
+///
+/// Please note that in rust, we need to differentiate between the [`BaseAudioContext`] trait and
+/// the [`ConcreteBaseAudioContext`] concrete implementation.
 #[allow(clippy::module_name_repetitions)]
-pub trait AsBaseAudioContext {
-    /// retrieves the `BaseAudioContext` associated with the concrete `AudioContext`
-    fn base(&self) -> &BaseAudioContext;
+pub trait BaseAudioContext {
+    /// retrieves the `ConcreteBaseAudioContext` associated with this `AudioContext`
+    fn base(&self) -> &ConcreteBaseAudioContext;
 
-    /// Retrieves an [`AudioBuffer`] from a given [`std::fs::File`]
+    /// Decode an [`AudioBuffer`] from a given input stream.
     ///
-    /// *Warning:* in the current implementation `decode_audio_data` only accepts
-    /// `.wav` files and do not perform any resampling operation. Be carefull
-    /// to use audio files that match the current sample rate or the file will
-    /// pitched when played.
-    fn decode_audio_data(&self, file: std::fs::File) -> AudioBuffer {
-        // @see - https://github.com/orottier/web-audio-api-rs/issues/42
-        // @note - should also be async, but that's not a big deal neither for now
-        let buf_reader = std::io::BufReader::new(file);
-        let mut reader = hound::WavReader::new(buf_reader).unwrap();
-        let hound::WavSpec {
-            channels,
-            sample_rate: _, // @todo - use for resampling
-            bits_per_sample,
-            sample_format,
-        } = reader.spec();
+    /// The current implementation can decode FLAC, Opus, PCM, Vorbis, and Wav.
+    ///
+    /// In addition to the official spec, the input parameter can be any byte stream (not just an
+    /// array). This means you can decode audio data from a file, network stream, or in memory
+    /// buffer, and any other [`std::io::Read`] implementor. The data if buffered internally so you
+    /// should not wrap the source in a `BufReader`.
+    ///
+    /// This function operates synchronously, which may be undesirable on the control thread. The
+    /// example shows how to avoid this. An async version is currently not implemented.
+    ///
+    /// # Errors
+    ///
+    /// This method returns an Error in various cases (IO, mime sniffing, decoding).
+    ///
+    /// # Example
+    ///
+    /// ```no_run
+    /// use std::io::Cursor;
+    /// use web_audio_api::SampleRate;
+    /// use web_audio_api::context::{BaseAudioContext, OfflineAudioContext};
+    ///
+    /// let input = Cursor::new(vec![0; 32]); // or a File, TcpStream, ...
+    ///
+    /// let context = OfflineAudioContext::new(2, 44_100, SampleRate(44_100));
+    /// let handle = std::thread::spawn(move || context.decode_audio_data_sync(input));
+    ///
+    /// // do other things
+    ///
+    /// // await result from the decoder thread
+    /// let decode_buffer_result = handle.join();
+    /// ```
+    fn decode_audio_data_sync<R: std::io::Read + Send + 'static>(
+        &self,
+        input: R,
+    ) -> Result<AudioBuffer, Box<dyn std::error::Error + Send + Sync>> {
+        // Set up a media decoder, consume the stream in full and construct a single buffer out of it
+        let mut buffer = MediaDecoder::try_new(input)?
+            .collect::<Result<Vec<_>, _>>()?
+            .into_iter()
+            .reduce(|mut accum, item| {
+                accum.extend(&item);
+                accum
+            })
+            // if there are no samples decoded, return an empty buffer
+            .unwrap_or_else(|| AudioBuffer::from(vec![vec![]], self.sample_rate_raw()));
 
-        let number_of_channels = channels as usize;
-        let length = reader.duration() as usize;
+        // resample to desired rate (no-op if already matching)
+        buffer.resample(self.sample_rate_raw());
 
-        let mut decoded: Vec<Vec<f32>> = Vec::with_capacity(number_of_channels);
-
-        for _ in 0..number_of_channels {
-            decoded.push(Vec::<f32>::with_capacity(length));
-        }
-
-        // @note - hound retrieves interleaved values
-        // cf. https://docs.rs/hound/latest/hound/struct.WavReader.html#method.samples
-        match sample_format {
-            hound::SampleFormat::Int => {
-                let mut channel_number = 0;
-
-                if bits_per_sample == 16 {
-                    for sample in reader.samples::<i16>() {
-                        let s = f32::from(sample.unwrap()) / f32::from(i16::MAX);
-                        assert!(s.abs() <= 1.);
-                        decoded[channel_number].push(s);
-                        // next sample belongs to the next channel
-                        channel_number = (channel_number + 1) % number_of_channels;
-                    }
-                } else {
-                    panic!("bits_per_sample {:?} not implemented", bits_per_sample);
-                }
-            }
-            hound::SampleFormat::Float => {
-                // this one is not tested
-                let mut channel_number = 0;
-
-                for sample in reader.samples::<f32>() {
-                    let s = sample.unwrap();
-                    decoded[channel_number].push(s);
-                    // next sample belongs to the next channel
-                    channel_number = (channel_number + 1) % number_of_channels;
-                }
-            }
-        }
-
-        assert_eq!(decoded[0].len(), length);
-
-        // [spec] Take the result, representing the decoded linear PCM audio data,
-        // and resample it to the sample-rate of the BaseAudioContext if it is
-        // different from the sample-rate of audioData.
-
-        // @todo - resample if needed
-        // if (sample_rate != self.sample_rate()) {}
-
-        let mut channels = Vec::<ChannelData>::with_capacity(number_of_channels);
-
-        // replace raw decoded channels with `DataChannel`
-        for raw_channel in decoded {
-            let channel = ChannelData::from(raw_channel);
-            channels.push(channel);
-        }
-
-        AudioBuffer::from_channels(channels, self.sample_rate())
+        Ok(buffer)
     }
 
     /// Create an new "in-memory" `AudioBuffer` with the given number of channels,
@@ -180,25 +167,56 @@ pub trait AsBaseAudioContext {
         AudioBuffer::new(options)
     }
 
-    /// Creates an `OscillatorNode`, a source representing a periodic waveform. It basically
-    /// generates a tone.
-    fn create_oscillator(&self) -> node::OscillatorNode {
-        node::OscillatorNode::new(self.base(), None)
+    /// Creates a `AnalyserNode`
+    fn create_analyser(&self) -> node::AnalyserNode {
+        node::AnalyserNode::new(self.base(), node::AnalyserOptions::default())
     }
 
-    /// Creates an `StereoPannerNode` to pan a stereo output
-    fn create_stereo_panner(&self) -> node::StereoPannerNode {
-        node::StereoPannerNode::new(self.base(), None)
+    /// Creates an `BiquadFilterNode` which implements a second order filter
+    fn create_biquad_filter(&self) -> node::BiquadFilterNode {
+        node::BiquadFilterNode::new(self.base(), node::BiquadFilterOptions::default())
     }
 
-    /// Creates an `GainNode`, to control audio volume
-    fn create_gain(&self) -> node::GainNode {
-        node::GainNode::new(self.base(), GainOptions::default())
+    /// Creates an `AudioBufferSourceNode`
+    fn create_buffer_source(&self) -> node::AudioBufferSourceNode {
+        node::AudioBufferSourceNode::new(self.base(), node::AudioBufferSourceOptions::default())
     }
 
     /// Creates an `ConstantSourceNode`, a source representing a constant value
     fn create_constant_source(&self) -> node::ConstantSourceNode {
-        node::ConstantSourceNode::new(self.base(), ConstantSourceOptions::default())
+        node::ConstantSourceNode::new(self.base(), node::ConstantSourceOptions::default())
+    }
+
+    /// Creates a `ChannelMergerNode`
+    fn create_channel_merger(&self, number_of_inputs: u32) -> node::ChannelMergerNode {
+        let opts = node::ChannelMergerOptions {
+            number_of_inputs,
+            ..node::ChannelMergerOptions::default()
+        };
+        node::ChannelMergerNode::new(self.base(), opts)
+    }
+
+    /// Creates a `ChannelSplitterNode`
+    fn create_channel_splitter(&self, number_of_outputs: u32) -> node::ChannelSplitterNode {
+        let opts = node::ChannelSplitterOptions {
+            number_of_outputs,
+            ..node::ChannelSplitterOptions::default()
+        };
+        node::ChannelSplitterNode::new(self.base(), opts)
+    }
+
+    /// Creates a `DelayNode`, delaying the audio signal
+    fn create_delay(&self, max_delay_time: f64) -> node::DelayNode {
+        let opts = node::DelayOptions {
+            max_delay_time,
+            ..node::DelayOptions::default()
+        };
+        node::DelayNode::new(self.base(), opts)
+    }
+
+    /// Creates an `GainNode`, to control audio volume
+    fn create_gain(&self) -> node::GainNode {
+        node::GainNode::new(self.base(), node::GainOptions::default())
     }
 
     /// Creates an `IirFilterNode`
@@ -209,67 +227,13 @@ pub trait AsBaseAudioContext {
     /// The maximum length of this array is 20
     /// * `feedback` - An array of the feedback (denominator) coefficients for the transfer function of the IIR filter.
     /// The maximum length of this array is 20
-    fn create_iir_filter(&self, feedforward: Vec<f64>, feedback: Vec<f64>) -> node::IirFilterNode {
-        let options = IirFilterOptions {
+    fn create_iir_filter(&self, feedforward: Vec<f64>, feedback: Vec<f64>) -> node::IIRFilterNode {
+        let options = node::IIRFilterOptions {
             channel_config: ChannelConfigOptions::default(),
             feedforward,
             feedback,
         };
-        node::IirFilterNode::new(self.base(), options)
-    }
-
-    /// Creates a `DelayNode`, delaying the audio signal
-    fn create_delay(&self, max_delay_time: f32) -> node::DelayNode {
-        let opts = node::DelayOptions {
-            max_delay_time,
-            ..DelayOptions::default()
-        };
-        node::DelayNode::new(self.base(), opts)
-    }
-
-    /// Creates an `BiquadFilterNode` which implements a second order filter
-    fn create_biquad_filter(&self) -> node::BiquadFilterNode {
-        node::BiquadFilterNode::new(self.base(), None)
-    }
-
-    /// Creates a `WaveShaperNode`
-    fn create_wave_shaper(&self) -> node::WaveShaperNode {
-        node::WaveShaperNode::new(self.base(), None)
-    }
-
-    /// Creates a `ChannelSplitterNode`
-    fn create_channel_splitter(&self, number_of_outputs: u32) -> node::ChannelSplitterNode {
-        let opts = node::ChannelSplitterOptions {
-            number_of_outputs,
-            ..ChannelSplitterOptions::default()
-        };
-        node::ChannelSplitterNode::new(self.base(), opts)
-    }
-
-    /// Creates a `ChannelMergerNode`
-    fn create_channel_merger(&self, number_of_inputs: u32) -> node::ChannelMergerNode {
-        let opts = node::ChannelMergerOptions {
-            number_of_inputs,
-            ..ChannelMergerOptions::default()
-        };
-        node::ChannelMergerNode::new(self.base(), opts)
-    }
-
-    /// Creates a `MediaStreamAudioSourceNode` from a `MediaElement`
-    fn create_media_stream_source<M: MediaStream>(
-        &self,
-        media: M,
-    ) -> node::MediaStreamAudioSourceNode {
-        let channel_config = ChannelConfigOptions {
-            count: 1,
-            mode: ChannelCountMode::Explicit,
-            interpretation: ChannelInterpretation::Speakers,
-        };
-        let opts = node::MediaStreamAudioSourceNodeOptions {
-            media,
-            channel_config,
-        };
-        node::MediaStreamAudioSourceNode::new(self.base(), opts)
+        node::IIRFilterNode::new(self.base(), options)
     }
 
     /// Creates a `MediaElementAudioSourceNode` from a `MediaElement`
@@ -279,46 +243,54 @@ pub trait AsBaseAudioContext {
         &self,
         media: MediaElement,
     ) -> node::MediaElementAudioSourceNode {
-        let channel_config = ChannelConfigOptions {
-            count: 1,
-            mode: ChannelCountMode::Explicit,
-            interpretation: ChannelInterpretation::Speakers,
-        };
-        let opts = node::MediaElementAudioSourceNodeOptions {
-            media,
-            channel_config,
+        let opts = node::MediaElementAudioSourceOptions {
+            media_element: media,
         };
         node::MediaElementAudioSourceNode::new(self.base(), opts)
     }
 
-    /// Creates an `AudioBufferSourceNode`
-    ///
-    /// Note: do not forget to `start()` the node.
-    fn create_buffer_source(&self) -> node::AudioBufferSourceNode {
-        node::AudioBufferSourceNode::new(self.base(), AudioBufferSourceOptions::default())
+    /// Creates a `MediaStreamAudioSourceNode` from a `MediaElement`
+    fn create_media_stream_source<M: MediaStream>(
+        &self,
+        media: M,
+    ) -> node::MediaStreamAudioSourceNode {
+        let opts = node::MediaStreamAudioSourceOptions {
+            media_stream: media,
+        };
+        node::MediaStreamAudioSourceNode::new(self.base(), opts)
+    }
+
+    /// Creates an `OscillatorNode`, a source representing a periodic waveform.
+    fn create_oscillator(&self) -> node::OscillatorNode {
+        node::OscillatorNode::new(self.base(), node::OscillatorOptions::default())
     }
 
     /// Creates a `PannerNode`
     fn create_panner(&self) -> node::PannerNode {
-        node::PannerNode::new(self.base(), PannerOptions::default())
-    }
-
-    /// Creates a `AnalyserNode`
-    fn create_analyser(&self) -> node::AnalyserNode {
-        node::AnalyserNode::new(self.base(), AnalyserOptions::default())
+        node::PannerNode::new(self.base(), node::PannerOptions::default())
     }
 
     /// Creates a periodic wave
-    fn create_periodic_wave(&self, options: Option<PeriodicWaveOptions>) -> PeriodicWave {
+    fn create_periodic_wave(&self, options: PeriodicWaveOptions) -> PeriodicWave {
         PeriodicWave::new(self.base(), options)
     }
 
+    /// Creates an `StereoPannerNode` to pan a stereo output
+    fn create_stereo_panner(&self) -> node::StereoPannerNode {
+        node::StereoPannerNode::new(self.base(), node::StereoPannerOptions::default())
+    }
+
+    /// Creates a `WaveShaperNode`
+    fn create_wave_shaper(&self) -> node::WaveShaperNode {
+        node::WaveShaperNode::new(self.base(), node::WaveShaperOptions::default())
+    }
+
     /// Create an `AudioParam`.
     ///
     /// Call this inside the `register` closure when setting up your `AudioNode`
     fn create_audio_param(
         &self,
-        opts: AudioParamOptions,
+        opts: AudioParamDescriptor,
         dest: &AudioNodeId,
     ) -> (crate::param::AudioParam, AudioParamId) {
         let param = self.base().register(move |registration| {
@@ -336,12 +308,12 @@ pub trait AsBaseAudioContext {
 
     /// Returns an `AudioDestinationNode` representing the final destination of all audio in the
     /// context. It can be thought of as the audio-rendering device.
-    fn destination(&self) -> node::DestinationNode {
+    fn destination(&self) -> node::AudioDestinationNode {
         let registration = AudioContextRegistration {
             id: AudioNodeId(DESTINATION_NODE_ID),
             context: self.base().clone(),
         };
-        node::DestinationNode {
+        node::AudioDestinationNode {
             registration,
             channel_count: self.base().channels() as usize,
         }
@@ -369,10 +341,18 @@ pub trait AsBaseAudioContext {
     }
 
     /// The sample rate (in sample-frames per second) at which the `AudioContext` handles audio.
-    fn sample_rate(&self) -> SampleRate {
+    #[must_use]
+    fn sample_rate(&self) -> f32 {
         self.base().sample_rate()
     }
 
+    /// The raw sample rate of the `AudioContext` (which has more precision than the float
+    /// [`sample_rate()`](BaseAudioContext::sample_rate) value).
+    #[must_use]
+    fn sample_rate_raw(&self) -> SampleRate {
+        self.base().sample_rate_raw()
+    }
+
     /// This is the time in seconds of the sample frame immediately following the last sample-frame
     /// in the block of audio most recently processed by the context’s rendering graph.
     fn current_time(&self) -> f64 {
@@ -388,15 +368,15 @@ pub trait AsBaseAudioContext {
     }
 }
 
-impl AsBaseAudioContext for BaseAudioContext {
-    fn base(&self) -> &BaseAudioContext {
+impl BaseAudioContext for ConcreteBaseAudioContext {
+    fn base(&self) -> &ConcreteBaseAudioContext {
         self
     }
 }
 
 /// Identify the type of playback, which affects tradeoffs
 /// between audio output latency and power consumption
-pub enum LatencyHint {
+pub enum AudioContextLatencyCategory {
     /// Balance audio output latency and power consumption.
     Balanced,
     /// Provide the lowest audio output latency possible without glitching. This is the default.
@@ -416,7 +396,7 @@ pub enum LatencyHint {
 pub struct AudioContextOptions {
     /// Identify the type of playback, which affects
     /// tradeoffs between audio output latency and power consumption
-    pub latency_hint: Option<LatencyHint>,
+    pub latency_hint: Option<AudioContextLatencyCategory>,
     /// Sample rate of the audio Context and audio output hardware
     pub sample_rate: Option<u32>,
     /// Number of output channels of destination node and audio output hardware
@@ -429,15 +409,15 @@ pub struct AudioContextOptions {
 #[allow(clippy::module_name_repetitions)]
 pub struct AudioContext {
     /// represents the underlying `BaseAudioContext`
-    base: BaseAudioContext,
+    base: ConcreteBaseAudioContext,
 
     /// cpal stream (play/pause functionality)
     #[cfg(not(test))] // in tests, do not set up a cpal Stream
-    stream: Stream,
+    stream: Mutex<Option<Stream>>,
 }
 
-impl AsBaseAudioContext for AudioContext {
-    fn base(&self) -> &BaseAudioContext {
+impl BaseAudioContext for AudioContext {
+    fn base(&self) -> &ConcreteBaseAudioContext {
         &self.base
     }
 }
@@ -448,15 +428,15 @@ impl AsBaseAudioContext for AudioContext {
 #[allow(clippy::module_name_repetitions)]
 pub struct OfflineAudioContext {
     /// represents the underlying `BaseAudioContext`
-    base: BaseAudioContext,
+    base: ConcreteBaseAudioContext,
     /// the size of the buffer in sample-frames
     length: usize,
     /// the rendering 'thread', fully controlled by the offline context
     renderer: RenderThread,
 }
 
-impl AsBaseAudioContext for OfflineAudioContext {
-    fn base(&self) -> &BaseAudioContext {
+impl BaseAudioContext for OfflineAudioContext {
+    fn base(&self) -> &ConcreteBaseAudioContext {
         &self.base
     }
 }
@@ -477,16 +457,19 @@ impl AudioContext {
         let channels = u32::from(config.channels);
         let sample_rate = SampleRate(config.sample_rate.0);
 
-        let base = BaseAudioContext::new(sample_rate, channels, frames_played, sender);
+        let base = ConcreteBaseAudioContext::new(sample_rate, channels, frames_played, sender);
 
-        Self { base, stream }
+        Self {
+            base,
+            stream: Mutex::new(Some(stream)),
+        }
     }
 
     #[cfg(test)] // in tests, do not set up a cpal Stream
     #[allow(clippy::must_use_candidate)]
     pub fn new(options: Option<AudioContextOptions>) -> Self {
         let options = options.unwrap_or(AudioContextOptions {
-            latency_hint: Some(LatencyHint::Interactive),
+            latency_hint: Some(AudioContextLatencyCategory::Interactive),
             sample_rate: Some(44_100),
             channels: Some(2),
         });
@@ -495,13 +478,18 @@ impl AudioContext {
         let channels = u32::from(options.channels.unwrap_or(2));
         let (sender, _receiver) = crossbeam_channel::unbounded();
         let frames_played = Arc::new(AtomicU64::new(0));
-        let base = BaseAudioContext::new(sample_rate, channels, frames_played, sender);
+        let base = ConcreteBaseAudioContext::new(sample_rate, channels, frames_played, sender);
 
         Self { base }
     }
 
-    /// Suspends the progression of time in the audio context, temporarily halting audio hardware
-    /// access and reducing CPU/battery usage in the process.
+    /// Suspends the progression of time in the audio context.
+    ///
+    /// This will temporarily halt audio hardware access and reducing CPU/battery usage in the
+    /// process.
+    ///
+    /// This function operates synchronously and might block the current thread. An async version
+    /// is currently not implemented.
     ///
     /// # Panics
     ///
@@ -511,14 +499,21 @@ impl AudioContext {
     /// * For a `BackendSpecificError`
     // false positive due to #[cfg(not(test))]
     #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
-    pub fn suspend(&self) {
+    pub fn suspend_sync(&self) {
         #[cfg(not(test))] // in tests, do not set up a cpal Stream
-        self.stream.pause().unwrap();
+        if let Some(s) = self.stream.lock().unwrap().as_ref() {
+            if let Err(e) = s.pause() {
+                panic!("Error suspending cpal stream: {:?}", e);
+            }
+        }
     }
 
     /// Resumes the progression of time in an audio context that has previously been
     /// suspended/paused.
     ///
+    /// This function operates synchronously and might block the current thread. An async version
+    /// is currently not implemented.
+    ///
     /// # Panics
     ///
     /// Will panic if:
@@ -527,9 +522,31 @@ impl AudioContext {
     /// * For a `BackendSpecificError`
     // false positive due to #[cfg(not(test))]
     #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
-    pub fn resume(&self) {
+    pub fn resume_sync(&self) {
         #[cfg(not(test))] // in tests, do not set up a cpal Stream
-        self.stream.play().expect("Audio device refuse to play");
+        if let Some(s) = self.stream.lock().unwrap().as_ref() {
+            if let Err(e) = s.play() {
+                panic!("Error resuming cpal stream: {:?}", e);
+            }
+        }
+    }
+
+    /// Closes the `AudioContext`, releasing the system resources being used.
+    ///
+    /// This will not automatically release all `AudioContext`-created objects, but will suspend
+    /// the progression of the currentTime, and stop processing audio data.
+    ///
+    /// This function operates synchronously and might block the current thread. An async version
+    /// is currently not implemented.
+    ///
+    /// # Panics
+    ///
+    /// Will panic when this function is called multiple times
+    // false positive due to #[cfg(not(test))]
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
+    pub fn close_sync(&self) {
+        #[cfg(not(test))] // in tests, do not set up a cpal Stream
+        self.stream.lock().unwrap().take(); // will Drop
     }
 }
 
@@ -555,10 +572,10 @@ impl From<&AudioParamId> for NodeIndex {
 ///
 /// This allows for communication with the render thread and lifetime management.
 ///
-/// The only way to construct this object is by calling [`BaseAudioContext::register`]
+/// The only way to construct this object is by calling [`ConcreteBaseAudioContext::register`]
 pub struct AudioContextRegistration {
     /// the audio context in wich nodes and connections lives
-    context: BaseAudioContext,
+    context: ConcreteBaseAudioContext,
     /// identify a specific `AudioNode`
     id: AudioNodeId,
 }
@@ -575,7 +592,7 @@ impl AudioContextRegistration {
     // false positive: AudioContextRegistration is not const
     #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
     #[must_use]
-    pub fn context(&self) -> &BaseAudioContext {
+    pub fn context(&self) -> &ConcreteBaseAudioContext {
         &self.context
     }
 }
@@ -589,12 +606,15 @@ impl Drop for AudioContextRegistration {
 
         if !magic {
             let message = ControlMessage::FreeWhenFinished { id: self.id.0 };
-            self.context.inner.render_channel.send(message).unwrap();
+
+            // Sending the message will fail when the render thread has already shut down.
+            // This is fine
+            let _r = self.context.inner.render_channel.send(message);
         }
     }
 }
 
-impl BaseAudioContext {
+impl ConcreteBaseAudioContext {
     /// Creates a `BaseAudioContext` instance
     fn new(
         sample_rate: SampleRate,
@@ -602,7 +622,7 @@ impl BaseAudioContext {
         frames_played: Arc<AtomicU64>,
         render_channel: Sender<ControlMessage>,
     ) -> Self {
-        let base_inner = BaseAudioContextInner {
+        let base_inner = ConcreteBaseAudioContextInner {
             sample_rate,
             channels,
             render_channel,
@@ -620,7 +640,7 @@ impl BaseAudioContext {
             // will create a cyclic reference, but we can reconstruct a new instance on the fly
             // when requested
 
-            let dest = node::DestinationNode::new(&base, channels as usize);
+            let dest = node::AudioDestinationNode::new(&base, channels as usize);
             let listener = crate::spatial::AudioListenerNode::new(&base);
 
             // hack: Connect the listener to the destination node to force it to render at each
@@ -662,8 +682,16 @@ impl BaseAudioContext {
     }
 
     /// The sample rate (in sample-frames per second) at which the `AudioContext` handles audio.
+    #[allow(clippy::cast_precision_loss)]
     #[must_use]
-    pub fn sample_rate(&self) -> SampleRate {
+    pub fn sample_rate(&self) -> f32 {
+        self.inner.sample_rate.0 as f32
+    }
+
+    /// The raw sample rate of the `AudioContext` (which has more precision than the float
+    /// [`sample_rate()`](BaseAudioContext::sample_rate) value).
+    #[must_use]
+    pub fn sample_rate_raw(&self) -> SampleRate {
         self.inner.sample_rate
     }
 
@@ -832,7 +860,7 @@ impl OfflineAudioContext {
         );
 
         // first, setup the base audio context
-        let base = BaseAudioContext::new(sample_rate, channels, frames_played, sender);
+        let base = ConcreteBaseAudioContext::new(sample_rate, channels, frames_played, sender);
 
         Self {
             base,
@@ -841,9 +869,11 @@ impl OfflineAudioContext {
         }
     }
 
-    /// `OfflineAudioContext` doesn't start rendering automatically
-    /// You need to call this function to start the audio rendering
-    pub fn start_rendering(&mut self) -> AudioBuffer {
+    /// Given the current connections and scheduled changes, starts rendering audio.
+    ///
+    /// This function will block the current thread and returns the rendered `AudioBuffer`
+    /// synchronously. An async version is currently not implemented.
+    pub fn start_rendering_sync(&mut self) -> AudioBuffer {
         // make buffer_size always a multiple of RENDER_QUANTUM_SIZE, so we can still render piecewise with
         // the desired number of frames.
         let buffer_size =
@@ -866,6 +896,7 @@ impl OfflineAudioContext {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use float_eq::assert_float_eq;
 
     fn require_send_sync_static<T: Send + Sync + 'static>(_: T) {}
 
@@ -879,32 +910,44 @@ mod tests {
         require_send_sync_static(registration);
     }
 
+    #[test]
+    fn test_sample_rate() {
+        let context = OfflineAudioContext::new(1, 0, SampleRate(96000));
+        assert_float_eq!(context.sample_rate(), 96000., abs_all <= 0.);
+        assert_eq!(context.sample_rate_raw(), SampleRate(96000));
+    }
+
     #[test]
     fn test_decode_audio_data() {
-        // @todo - complete and assert when resampling will be implemented
-        let context = AudioContext::new(None);
+        let context = OfflineAudioContext::new(1, 0, SampleRate(44100));
+        let file = std::fs::File::open("samples/sample.wav").unwrap();
+        let audio_buffer = context.decode_audio_data_sync(file).unwrap();
 
-        let file = std::fs::File::open("sample.wav").unwrap();
-        let audio_buffer = context.decode_audio_data(file);
-
-        println!("----------------------------------------------");
-        println!(
-            "- number_of_channels: {:?}",
-            audio_buffer.number_of_channels()
-        );
-        println!("- length: {:?}", audio_buffer.length());
-        println!("- sample_rate: {:?}", audio_buffer.sample_rate());
-        println!("- duration: {:?}", audio_buffer.duration());
+        assert_eq!(audio_buffer.sample_rate_raw(), SampleRate(44100));
+        assert_eq!(audio_buffer.length(), 142_187);
+        assert_eq!(audio_buffer.number_of_channels(), 2);
+        assert_float_eq!(audio_buffer.duration(), 3.224, abs_all <= 0.001);
 
         let left_start = &audio_buffer.get_channel_data(0)[0..100];
         let right_start = &audio_buffer.get_channel_data(1)[0..100];
+        // assert distinct two channel data
+        assert!(left_start != right_start);
+    }
 
-        println!("----------------------------------------------");
-        println!("@todo - should check that resampling is ok    ");
-        println!("----------------------------------------------");
-        println!("- left_start: {:?}", left_start);
-        println!("----------------------------------------------");
-        println!("- right_start: {:?}", right_start);
-        println!("----------------------------------------------");
+    // #[test]
+    // disabled: symphonia cannot handle empty WAV-files
+    #[allow(dead_code)]
+    fn test_decode_audio_data_empty() {
+        let context = OfflineAudioContext::new(1, 0, SampleRate(44100));
+        let file = std::fs::File::open("samples/empty_2c.wav").unwrap();
+        let audio_buffer = context.decode_audio_data_sync(file).unwrap();
+        assert_eq!(audio_buffer.length(), 0);
+    }
+
+    #[test]
+    fn test_decode_audio_data_decoding_error() {
+        let context = OfflineAudioContext::new(1, 0, SampleRate(44100));
+        let file = std::fs::File::open("samples/corrupt.wav").unwrap();
+        assert!(context.decode_audio_data_sync(file).is_err());
     }
 }
