@@ -21,7 +21,7 @@ use cpal::{
 };
 
 use crate::buffer::AudioBuffer;
-use crate::context::{AudioContextOptions, LatencyHint};
+use crate::context::{AudioContextLatencyCategory, AudioContextOptions};
 use crate::media::MicrophoneRender;
 use crate::render::RenderThread;
 
@@ -151,8 +151,9 @@ impl StreamConfigsBuilder {
                 None => (default_buffer_size + buffer_size - 1) / buffer_size * buffer_size,
                 Some(l) => {
                     return match l {
-                        LatencyHint::Interactive => default_buffer_size,
-                        LatencyHint::Balanced => match self.supported.buffer_size() {
+                        AudioContextLatencyCategory::Interactive => default_buffer_size,
+                        AudioContextLatencyCategory::Balanced => match self.supported.buffer_size()
+                        {
                             SupportedBufferSize::Range { max, .. } => {
                                 let b = (default_buffer_size * 2).min(*max);
                                 (b + buffer_size - 1) / buffer_size * buffer_size
@@ -162,7 +163,8 @@ impl StreamConfigsBuilder {
                                 (b + buffer_size - 1) / buffer_size * buffer_size
                             }
                         },
-                        LatencyHint::Playback => match self.supported.buffer_size() {
+                        AudioContextLatencyCategory::Playback => match self.supported.buffer_size()
+                        {
                             SupportedBufferSize::Range { max, .. } => {
                                 let b = (default_buffer_size * 4).min(*max);
                                 (b + buffer_size - 1) / buffer_size * buffer_size
@@ -176,7 +178,7 @@ impl StreamConfigsBuilder {
                         #[allow(clippy::cast_sign_loss)]
                         // truncation is the desired behavior
                         #[allow(clippy::cast_possible_truncation)]
-                        LatencyHint::Specific(t) => {
+                        AudioContextLatencyCategory::Specific(t) => {
                             let b = t * f64::from(self.prefered.sample_rate.0);
                             (b as u32 + buffer_size - 1) / buffer_size * buffer_size
                         }
