@@ -1,15 +1,26 @@
 use crossbeam_channel::{Receiver, Sender};
+use once_cell::sync::OnceCell;
+use std::sync::atomic::{AtomicBool, Ordering};
 
 use crate::buffer::AudioBuffer;
-use crate::context::{AsBaseAudioContext, AudioContextRegistration, AudioParamId};
+use crate::context::{AudioContextRegistration, AudioParamId, BaseAudioContext};
 use crate::control::Controller;
-use crate::param::{AudioParam, AudioParamOptions, AutomationRate};
+use crate::param::{AudioParam, AudioParamDescriptor, AutomationRate};
 use crate::render::{AudioParamValues, AudioProcessor, AudioRenderQuantum};
 use crate::{SampleRate, RENDER_QUANTUM_SIZE};
 
 use super::{AudioNode, ChannelConfig, ChannelConfigOptions};
 
 /// Options for constructing an [`AudioBufferSourceNode`]
+// dictionary AudioBufferSourceOptions {
+//   AudioBuffer? buffer;
+//   float detune = 0;
+//   boolean loop = false;
+//   double loopEnd = 0;
+//   double loopStart = 0;
+//   float playbackRate = 1;
+// };
+#[derive(Clone, Debug)]
 pub struct AudioBufferSourceOptions {
     pub buffer: Option<AudioBuffer>,
     pub detune: f32,
@@ -29,7 +40,7 @@ impl Default for AudioBufferSourceOptions {
             loop_start: 0.,
             loop_end: 0.,
             playback_rate: 1.,
-            channel_config: Default::default(),
+            channel_config: ChannelConfigOptions::default(),
         }
     }
 }
@@ -42,23 +53,23 @@ struct AudioBufferMessage(AudioBuffer);
 ///
 /// - MDN documentation: <https://developer.mozilla.org/en-US/docs/Web/API/AudioBufferSourceNode>
 /// - specification: <https://webaudio.github.io/web-audio-api/#AudioBufferSourceNode>
-/// - see also: [`AsBaseAudioContext::create_buffer_source`](crate::context::AsBaseAudioContext::create_buffer_source)
+/// - see also: [`BaseAudioContext::create_buffer_source`](crate::context::BaseAudioContext::create_buffer_source)
 ///
 /// # Usage
 ///
 /// ```no_run
 /// use std::fs::File;
-/// use web_audio_api::context::{AsBaseAudioContext, AudioContext};
+/// use web_audio_api::context::{BaseAudioContext, AudioContext};
 /// use web_audio_api::node::AudioNode;
 ///
 /// // create an `AudioContext`
 /// let context = AudioContext::new(None);
 /// // load and decode a soundfile
-/// let file = File::open("sample.wav").unwrap();
-/// let audio_buffer = context.decode_audio_data(file);
+/// let file = File::open("samples/sample.wav").unwrap();
+/// let audio_buffer = context.decode_audio_data_sync(file).unwrap();
 /// // play the sound file
-/// let mut src = context.create_buffer_source();
-/// src.set_buffer(&audio_buffer);
+/// let src = context.create_buffer_source();
+/// src.set_buffer(audio_buffer);
 /// src.connect(&context.destination());
 /// src.start();
 /// ```
@@ -75,8 +86,8 @@ pub struct AudioBufferSourceNode {
     sender: Sender<AudioBufferMessage>,
     detune: AudioParam,        // has constraints, no a-rate
     playback_rate: AudioParam, // has constraints, no a-rate
-    buffer: Option<AudioBuffer>,
-    source_started: bool,
+    buffer: OnceCell<AudioBuffer>,
+    source_started: AtomicBool,
 }
 
 impl AudioNode for AudioBufferSourceNode {
@@ -91,6 +102,7 @@ impl AudioNode for AudioBufferSourceNode {
     fn number_of_inputs(&self) -> u32 {
         0
     }
+
     fn number_of_outputs(&self) -> u32 {
         1
     }
@@ -98,7 +110,7 @@ impl AudioNode for AudioBufferSourceNode {
 
 impl AudioBufferSourceNode {
     /// Create a new [`AudioBufferSourceNode`] instance
-    pub fn new<C: AsBaseAudioContext>(context: &C, options: AudioBufferSourceOptions) -> Self {
+    pub fn new<C: BaseAudioContext>(context: &C, options: AudioBufferSourceOptions) -> Self {
         context.base().register(move |registration| {
             let AudioBufferSourceOptions {
                 buffer,
@@ -113,7 +125,7 @@ impl AudioBufferSourceNode {
             // @todo - these parameters can't be changed to a-rate
             // @see - <https://webaudio.github.io/web-audio-api/#audioparam-automation-rate-constraints>
             // @see - https://github.com/orottier/web-audio-api-rs/issues/29
-            let detune_param_options = AudioParamOptions {
+            let detune_param_options = AudioParamDescriptor {
                 min_value: f32::MIN,
                 max_value: f32::MAX,
                 default_value: 0.,
@@ -125,7 +137,7 @@ impl AudioBufferSourceNode {
 
             d_param.set_value(detune);
 
-            let playback_rate_param_options = AudioParamOptions {
+            let playback_rate_param_options = AudioParamDescriptor {
                 min_value: f32::MIN,
                 max_value: f32::MAX,
                 default_value: 1.,
@@ -137,19 +149,8 @@ impl AudioBufferSourceNode {
 
             pr_param.set_value(playback_rate);
 
-            // Channel to send buffer channels references to the renderer
-            //
-            // @note: `crossbeam_channel::bounded(1)`this will block the control
-            // thread until the buffer is received. This could maybe be improved.
-            //
-            // Would it be feasible to spawn a middle man thread that could block?
-            // ```
-            // let (sender, receiver) = crossbeam_channel::bounded(0);
-            // // this thread could block waiting for the render thread
-            // thread::spawn(move || {
-            //     sender.send(data);
-            // });
-            // ```
+            // Channel to send buffer channels references to the renderer.
+            // A capacity of 1 suffices since it is not allowed to set the value multiple times
             let (sender, receiver) = crossbeam_channel::bounded(1);
 
             let controller = Controller::new();
@@ -161,22 +162,21 @@ impl AudioBufferSourceNode {
                 detune: d_proc,
                 playback_rate: pr_proc,
                 render_state: AudioBufferRendererState::default(),
-                // `internal_buffer` is used to compute the samples per channel
-                // at each frame. 2 channels are sufficient as default value for
-                // most use-cases, note that the `vec` will always be resized to
-                // actual buffer number_of_channels when received on the render thread.
-                internal_buffer: Vec::<f32>::with_capacity(2),
+                // `internal_buffer` is used to compute the samples per channel at each frame. Note
+                // that the `vec` will always be resized to actual buffer number_of_channels when
+                // received on the render thread.
+                internal_buffer: Vec::<f32>::with_capacity(crate::MAX_CHANNELS),
             };
 
-            let mut node = Self {
+            let node = Self {
                 registration,
                 controller,
                 channel_config: channel_config.into(),
                 sender,
                 detune: d_param,
                 playback_rate: pr_param,
-                buffer: None,
-                source_started: false,
+                buffer: OnceCell::new(),
+                source_started: AtomicBool::new(false),
             };
 
             node.controller.set_loop(loop_);
@@ -184,85 +184,80 @@ impl AudioBufferSourceNode {
             node.controller.set_loop_end(loop_end);
 
             if let Some(buf) = buffer {
-                node.set_buffer(&buf);
+                node.set_buffer(buf);
             }
 
             (node, Box::new(renderer))
         })
     }
 
+    /// Current buffer value (nullable)
+    pub fn buffer(&self) -> Option<&AudioBuffer> {
+        self.buffer.get()
+    }
+
     /// Provide an [`AudioBuffer`] as the source of data to be played bask
     ///
     /// # Panics
     ///
-    /// Panics if a buffer has already been given to the source (though `new`
-    /// or through `set_buffer`)
-    pub fn set_buffer(&mut self, audio_buffer: &AudioBuffer) {
-        // - Let new buffer be the AudioBuffer or null value to be assigned to buffer.
-        // - If new buffer is not null and [[buffer set]] is true, throw an
-        // InvalidStateError and abort these steps.
-        if self.buffer.is_some() {
+    /// Panics if a buffer has already been given to the source (though `new` or through
+    /// `set_buffer`)
+    pub fn set_buffer(&self, audio_buffer: AudioBuffer) {
+        let clone = audio_buffer.clone();
+
+        if self.buffer.set(audio_buffer).is_err() {
             panic!("InvalidStateError - cannot assign buffer twice");
         }
-        // - If new buffer is not null, set [[buffer set]] to true.
-        // - Assign new buffer to the buffer attribute.
-        self.buffer = Some(audio_buffer.clone());
-        // - If start() has previously been called on this node, perform the
-        // operation acquire the content on buffer.
-        // see <https://github.com/orottier/web-audio-api-rs/issues/68> for
-        // more information on the "acquire the content" concept in this context.
-        if self.source_started {
-            self.send_buffer();
-        }
+
+        self.sender
+            .send(AudioBufferMessage(clone))
+            .expect("Sending AudioBufferMessage failed");
     }
 
     /// Start the playback on next block
-    pub fn start(&mut self) {
+    pub fn start(&self) {
         let start = self.registration.context().current_time();
         self.start_at_with_offset_and_duration(start, 0., f64::MAX);
     }
 
     /// Start the playback at the given time
-    pub fn start_at(&mut self, start: f64) {
+    pub fn start_at(&self, start: f64) {
         self.start_at_with_offset_and_duration(start, 0., f64::MAX);
     }
 
     /// Start the playback at the given time and with a given offset
-    pub fn start_at_with_offset(&mut self, start: f64, offset: f64) {
+    pub fn start_at_with_offset(&self, start: f64, offset: f64) {
         self.start_at_with_offset_and_duration(start, offset, f64::MAX);
     }
 
     /// Start the playback at the given time, with a given offset, for a given duration
-    pub fn start_at_with_offset_and_duration(&mut self, start: f64, offset: f64, duration: f64) {
-        if self.source_started {
+    pub fn start_at_with_offset_and_duration(&self, start: f64, offset: f64, duration: f64) {
+        if self.source_started.swap(true, Ordering::SeqCst) {
             panic!("InvalidStateError: Cannot call `start` twice");
         }
 
-        self.source_started = true;
         self.controller.set_offset(offset);
         self.controller.set_duration(duration);
         self.controller.scheduler().start_at(start);
-
-        self.send_buffer();
     }
 
     /// Stop the playback on next block
-    pub fn stop(&mut self) {
+    pub fn stop(&self) {
         let stop = self.registration.context().current_time();
         self.stop_at(stop);
     }
 
     /// Stop the playback at given time
-    pub fn stop_at(&mut self, stop: f64) {
-        if !self.source_started {
+    pub fn stop_at(&self, stop: f64) {
+        if !self.source_started.load(Ordering::SeqCst) {
             panic!("InvalidStateError cannot stop before start");
         }
 
         self.controller.scheduler().stop_at(stop)
     }
 
-    /// [`AudioParam`] that defines the speed at which the [`AudioBuffer`] will
-    /// be played, e.g.:
+    /// K-rate [`AudioParam`] that defines the speed at which the [`AudioBuffer`]
+    /// will be played, e.g.:
     /// - `0.5` will play the file at half speed
     /// - `-1` will play the file in reverse
     ///
@@ -271,7 +266,8 @@ impl AudioBufferSourceNode {
         &self.playback_rate
     }
 
-    /// [`AudioParam`] that defines a pitch transposition of the file, expressed in cents
+    /// K-rate [`AudioParam`] that defines a pitch transposition of the file,
+    /// expressed in cents
     ///
     /// see <https://en.wikipedia.org/wiki/Cent_(music)>
     pub fn detune(&self) -> &AudioParam {
@@ -304,13 +300,6 @@ impl AudioBufferSourceNode {
     pub fn set_loop_end(&self, value: f64) {
         self.controller.set_loop_end(value);
     }
-
-    fn send_buffer(&self) {
-        let clone = self.buffer.as_ref().unwrap().clone();
-        self.sender
-            .send(AudioBufferMessage(clone))
-            .expect("Sending AudioBufferMessage failed");
-    }
 }
 
 struct AudioBufferRendererState {
@@ -350,12 +339,12 @@ impl AudioProcessor for AudioBufferSourceRenderer {
         timestamp: f64,
         sample_rate: SampleRate,
     ) -> bool {
-        // ... and single output node
+        // single output node
         let output = &mut outputs[0];
 
         let dt = 1. / sample_rate.0 as f64;
         let num_frames = RENDER_QUANTUM_SIZE;
-        let next_block_time = timestamp + dt * RENDER_QUANTUM_SIZE as f64;
+        let next_block_time = timestamp + dt * num_frames as f64;
 
         if let Ok(msg) = self.receiver.try_recv() {
             let buffer = msg.0;
@@ -369,10 +358,10 @@ impl AudioProcessor for AudioBufferSourceRenderer {
         let playback_rate_values = params.get(&self.playback_rate);
         let detune = detune_values[0];
         let playback_rate = playback_rate_values[0];
-        let computed_playback_rate = (playback_rate * 2_f32.powf(detune / 1200.)) as f64;
+        let computed_playback_rate = (playback_rate * (detune / 1200.).exp2()) as f64;
 
         // grab all timing informations
-        let start_time = self.controller.scheduler().get_start_at();
+        let mut start_time = self.controller.scheduler().get_start_at();
         let stop_time = self.controller.scheduler().get_stop_at();
         let mut offset = self.controller.offset();
         let duration = self.controller.duration();
@@ -392,36 +381,38 @@ impl AudioProcessor for AudioBufferSourceRenderer {
 
         // If the buffer has not been set wait for it.
         // @see - `set_buffer` tries to acquire the buffer if the source already started
-        if self.buffer.is_none() {
-            output.make_silent();
-            return true;
-        }
+        let buffer = match &self.buffer {
+            None => {
+                output.make_silent();
+                return true;
+            }
+            Some(b) => b,
+        };
 
         // from this point we know that we have a buffer
+        let buffer_duration = buffer.duration();
 
         // In addition, if the buffer has more than one channel, then the
         // AudioBufferSourceNode output must change to a single channel of silence
         // at the beginning of a render quantum after the time at which any one of
         // the following conditions holds:
-        let buffer = self.buffer.as_ref().unwrap();
-        let buffer_duration = buffer.duration();
 
         // 1. the stop time has been reached.
         // 2. the duration has been reached.
         if timestamp >= stop_time || self.render_state.buffer_time_elapsed >= duration {
-            output.make_silent();
+            output.make_silent(); // also converts to mono
             return false;
         }
 
         // 3. the end of the buffer has been reached.
         if !loop_ {
             if computed_playback_rate > 0. && self.render_state.buffer_time >= buffer_duration {
-                output.make_silent();
+                output.make_silent(); // also converts to mono
                 return false;
             }
 
             if computed_playback_rate < 0. && self.render_state.buffer_time < 0. {
-                output.make_silent();
+                output.make_silent(); // also converts to mono
                 return false;
             }
         }
@@ -432,6 +423,14 @@ impl AudioProcessor for AudioBufferSourceRenderer {
         // @see <https://webaudio.github.io/web-audio-api/#playback-AudioBufferSourceNode>
         let mut current_time = timestamp;
 
+        // prevent scheduling in the past
+        // If 0 is passed in for this value or if the value is less than
+        // currentTime, then the sound will start playing immediately
+        // cf. https://webaudio.github.io/web-audio-api/#dom-audioscheduledsourcenode-start-when-when
+        if !self.render_state.started && start_time < current_time {
+            start_time = current_time;
+        }
+
         if loop_ {
             if loop_start >= 0. && loop_end > 0. && loop_start < loop_end {
                 actual_loop_start = loop_start;
@@ -451,11 +450,15 @@ impl AudioProcessor for AudioBufferSourceRenderer {
             {
                 self.internal_buffer.fill(0.);
                 output.set_channels_values_at(index, &self.internal_buffer);
+                current_time += dt;
+
                 continue; // nothing more to do for this sample
             }
 
             // we have now reached start time
             if !self.render_state.started {
+                offset += current_time - start_time;
+
                 if loop_ && computed_playback_rate >= 0. && offset >= actual_loop_end {
                     offset = actual_loop_end;
                 }
@@ -566,7 +569,7 @@ impl AudioBufferSourceRenderer {
 
 #[cfg(test)]
 mod tests {
-    use crate::context::{AsBaseAudioContext, OfflineAudioContext};
+    use crate::context::{BaseAudioContext, OfflineAudioContext};
     use crate::node::AudioNode;
     use crate::{SampleRate, RENDER_QUANTUM_SIZE};
 
@@ -576,18 +579,16 @@ mod tests {
     fn test_playing_some_file() {
         let mut context = OfflineAudioContext::new(2, RENDER_QUANTUM_SIZE, SampleRate(44_100));
 
-        let file = std::fs::File::open("sample.wav").unwrap();
-        let audio_buffer = context.decode_audio_data(file);
+        let file = std::fs::File::open("samples/sample.wav").unwrap();
+        let audio_buffer = context.decode_audio_data_sync(file).unwrap();
 
-        let mut src = context.create_buffer_source();
-        src.set_buffer(&audio_buffer);
+        let src = context.create_buffer_source();
+        src.set_buffer(audio_buffer.clone());
         src.connect(&context.destination());
         src.start_at(context.current_time());
         src.stop_at(context.current_time() + 128.);
 
-        let res = context.start_rendering();
-        // println!("buffer duration: {:?}", audio_buffer.duration());
-        // println!("context sample rate: {:?}", context.sample_rate());
+        let res = context.start_rendering_sync();
 
         // check first 128 samples in left and right channels
         assert_float_eq!(
@@ -602,4 +603,122 @@ mod tests {
             abs_all <= 0.
         );
     }
+
+    #[test]
+    fn test_sub_quantum_start() {
+        let sample_rate = 128;
+        let sr = SampleRate(sample_rate as u32);
+        let mut context = OfflineAudioContext::new(1, sample_rate, sr);
+
+        let mut dirac = context.create_buffer(1, 1, sr);
+        dirac.copy_to_channel(&[1.], 0);
+
+        let src = context.create_buffer_source();
+        src.connect(&context.destination());
+        src.set_buffer(dirac);
+        src.start_at(1. / sample_rate as f64);
+
+        let result = context.start_rendering_sync();
+        let channel = result.get_channel_data(0);
+
+        let mut expected = vec![0.; sample_rate];
+        expected[1] = 1.;
+
+        assert_float_eq!(channel[..], expected[..], abs_all <= 0.);
+    }
+
+    #[test]
+    fn test_sub_sample_start() {
+        // sub sample
+        let sample_rate = 128;
+        let sr = SampleRate(sample_rate as u32);
+        let mut context = OfflineAudioContext::new(1, sample_rate, sr);
+
+        let mut dirac = context.create_buffer(1, sample_rate, sr);
+        dirac.copy_to_channel(&[1.], 0);
+
+        let src = context.create_buffer_source();
+        src.connect(&context.destination());
+        src.set_buffer(dirac);
+        src.start_at(1.5 / sample_rate as f64);
+
+        let result = context.start_rendering_sync();
+        let channel = result.get_channel_data(0);
+
+        let mut expected = vec![0.; sample_rate];
+        expected[2] = 0.5;
+
+        assert_float_eq!(channel[..], expected[..], abs_all <= 0.);
+    }
+
+    #[test]
+    fn test_sub_quantum_stop() {
+        let sample_rate = 128;
+        let sr = SampleRate(sample_rate as u32);
+        let mut context = OfflineAudioContext::new(1, sample_rate, sr);
+
+        let mut dirac = context.create_buffer(1, sample_rate, sr);
+        dirac.copy_to_channel(&[0., 0., 0., 0., 1.], 0);
+
+        let src = context.create_buffer_source();
+        src.connect(&context.destination());
+        src.set_buffer(dirac);
+        src.start_at(0. / sample_rate as f64);
+        // stop at time of dirac, shoud not be played
+        src.stop_at(4. / sample_rate as f64);
+
+        let result = context.start_rendering_sync();
+        let channel = result.get_channel_data(0);
+        let expected = vec![0.; sample_rate];
+
+        assert_float_eq!(channel[..], expected[..], abs_all <= 0.);
+    }
+
+    #[test]
+    fn test_sub_sample_stop() {
+        let sample_rate = 128;
+        let sr = SampleRate(sample_rate as u32);
+        let mut context = OfflineAudioContext::new(1, sample_rate, sr);
+
+        let mut dirac = context.create_buffer(1, sample_rate, sr);
+        dirac.copy_to_channel(&[0., 0., 0., 0., 1., 1.], 0);
+
+        let src = context.create_buffer_source();
+        src.connect(&context.destination());
+        src.set_buffer(dirac);
+        src.start_at(0. / sample_rate as f64);
+        // stop at between two diracs, only first one should be played
+        src.stop_at(4.5 / sample_rate as f64);
+
+        let result = context.start_rendering_sync();
+        let channel = result.get_channel_data(0);
+
+        let mut expected = vec![0.; sample_rate];
+        expected[4] = 1.;
+
+        assert_float_eq!(channel[..], expected[..], abs_all <= 0.);
+    }
+
+    #[test]
+    fn test_schedule_in_the_past() {
+        let sample_rate = 128;
+        let sr = SampleRate(sample_rate as u32);
+        let mut context = OfflineAudioContext::new(1, sample_rate, sr);
+
+        let mut dirac = context.create_buffer(1, 1, sr);
+        dirac.copy_to_channel(&[1.], 0);
+
+        let src = context.create_buffer_source();
+        src.connect(&context.destination());
+        src.set_buffer(dirac);
+        src.start_at(-1.);
+
+        let result = context.start_rendering_sync();
+        let channel = result.get_channel_data(0);
+
+        let mut expected = vec![0.; sample_rate];
+        expected[0] = 1.;
+
+        assert_float_eq!(channel[..], expected[..], abs_all <= 0.);
+    }
 }
