@@ -1,11 +1,4 @@
 //! The biquad filter control and renderer parts
-#![warn(
-    clippy::all,
-    clippy::pedantic,
-    clippy::nursery,
-    clippy::perf,
-    clippy::missing_docs_in_private_items
-)]
 use std::{
     f64::consts::{PI, SQRT_2},
     sync::{
@@ -19,8 +12,8 @@ use crossbeam_channel::{Receiver, Sender};
 use num_complex::Complex;
 
 use crate::{
-    context::{AsBaseAudioContext, AudioContextRegistration, AudioParamId},
-    param::{AudioParam, AudioParamOptions},
+    context::{AudioContextRegistration, AudioParamId, BaseAudioContext},
+    param::{AudioParam, AudioParamDescriptor},
     render::{AudioParamValues, AudioProcessor, AudioRenderQuantum},
     SampleRate, MAX_CHANNELS,
 };
@@ -34,8 +27,6 @@ struct CoeffsReq(Sender<[f64; 5]>);
 
 /// enumerates all the biquad filter types
 #[derive(Debug, Clone, Copy, PartialEq)]
-// the naming comes from the web audio specfication
-#[allow(clippy::module_name_repetitions)]
 pub enum BiquadFilterType {
     /// Allows frequencies below the cutoff frequency to pass through and attenuates frequencies above the cutoff. (12dB/oct rolloff)
     Lowpass,
@@ -81,45 +72,39 @@ impl From<u32> for BiquadFilterType {
     }
 }
 
-/// `BiquadFilterOptions` is used to pass options
-/// during the construction of `BiquadFilterNode` using its
-/// constructor method `new`
-// the naming comes from the web audio specfication
-#[allow(clippy::module_name_repetitions)]
+/// Options for constructing a [`BiquadFilterNode`]
+// dictionary BiquadFilterOptions : AudioNodeOptions {
+//   BiquadFilterType type = "lowpass";
+//   float Q = 1;
+//   float detune = 0;
+//   float frequency = 350;
+//   float gain = 0;
+// };
+#[derive(Clone, Debug)]
 pub struct BiquadFilterOptions {
-    /// The desired initial value for Q, if None default to 1.
-    pub q: Option<f32>,
-    /// The desired initial value for detune, if None default to 0.
-    pub detune: Option<f32>,
-    /// The desired initial value for frequency, if None default to 350.
-    pub frequency: Option<f32>,
-    /// The desired initial value for gain, if None default to 0.
-    pub gain: Option<f32>,
-    /// The desired initial value for type, if None default to Lowpass.
-    pub type_: Option<BiquadFilterType>,
-    /// audio node options
+    pub q: f32,
+    pub detune: f32,
+    pub frequency: f32,
+    pub gain: f32,
+    pub type_: BiquadFilterType,
     pub channel_config: ChannelConfigOptions,
 }
 
 impl Default for BiquadFilterOptions {
     fn default() -> Self {
         Self {
-            q: Some(1.),
-            detune: Some(0.),
-            frequency: Some(350.),
-            gain: Some(0.),
-            type_: Some(BiquadFilterType::default()),
+            q: 1.,
+            detune: 0.,
+            frequency: 350.,
+            gain: 0.,
+            type_: BiquadFilterType::default(),
             channel_config: ChannelConfigOptions::default(),
         }
     }
 }
 
 /// `BiquadFilterNode` is a second order IIR filter
-// the naming comes from the web audio specfication
-#[allow(clippy::module_name_repetitions)]
 pub struct BiquadFilterNode {
-    /// Sample rate (equals to audio context sample rate)
-    sample_rate: f32,
     /// Represents the node instance and its associated audio context
     registration: AudioContextRegistration,
     /// Infos about audio node channel configuration
@@ -154,6 +139,7 @@ impl AudioNode for BiquadFilterNode {
     fn number_of_inputs(&self) -> u32 {
         1
     }
+
     fn number_of_outputs(&self) -> u32 {
         1
     }
@@ -166,29 +152,18 @@ impl BiquadFilterNode {
     ///
     /// * `context` - audio context in which the audio node will live.
     /// * `options` - biquad filter options
-    pub fn new<C: AsBaseAudioContext>(context: &C, options: Option<BiquadFilterOptions>) -> Self {
+    pub fn new<C: BaseAudioContext>(context: &C, options: BiquadFilterOptions) -> Self {
         context.base().register(move |registration| {
-            let options = options.unwrap_or_default();
-            // cannot guarantee that the cast will be without loss of precision for all fs
-            // but for usual sample rate (44.1kHz, 48kHz, 96kHz) it is
-            #[allow(clippy::cast_precision_loss)]
-            let sample_rate = context.base().sample_rate().0 as f32;
+            let q_value = options.q;
+            let d_value = options.detune;
+            let f_value = options.frequency;
+            let g_value = options.gain;
+            let t_value = options.type_;
 
-            let default_freq = 350.;
-            let default_gain = 0.;
-            let default_det = 0.;
-            let default_q = 1.;
-
-            let q_value = options.q.unwrap_or(default_q);
-            let d_value = options.detune.unwrap_or(default_det);
-            let f_value = options.frequency.unwrap_or(default_freq);
-            let g_value = options.gain.unwrap_or(default_gain);
-            let t_value = options.type_.unwrap_or(BiquadFilterType::Lowpass);
-
-            let q_param_opts = AudioParamOptions {
+            let q_param_opts = AudioParamDescriptor {
                 min_value: f32::MIN,
                 max_value: f32::MAX,
-                default_value: default_q,
+                default_value: 1.,
                 automation_rate: crate::param::AutomationRate::A,
             };
             let (q_param, q_proc) = context
@@ -197,10 +172,10 @@ impl BiquadFilterNode {
 
             q_param.set_value(q_value);
 
-            let d_param_opts = AudioParamOptions {
+            let d_param_opts = AudioParamDescriptor {
                 min_value: -153_600.,
                 max_value: 153_600.,
-                default_value: default_det,
+                default_value: 0.,
                 automation_rate: crate::param::AutomationRate::A,
             };
             let (d_param, d_proc) = context
@@ -209,13 +184,11 @@ impl BiquadFilterNode {
 
             d_param.set_value(d_value);
 
-            let niquyst = context.base().sample_rate().0 / 2;
-            // It should be fine for usual fs
-            #[allow(clippy::cast_precision_loss)]
-            let f_param_opts = AudioParamOptions {
+            let niquyst = context.sample_rate() / 2.;
+            let f_param_opts = AudioParamDescriptor {
                 min_value: 0.,
-                max_value: niquyst as f32,
-                default_value: default_freq,
+                max_value: niquyst,
+                default_value: 350.,
                 automation_rate: crate::param::AutomationRate::A,
             };
             let (f_param, f_proc) = context
@@ -224,10 +197,10 @@ impl BiquadFilterNode {
 
             f_param.set_value(f_value);
 
-            let g_param_opts = AudioParamOptions {
+            let g_param_opts = AudioParamDescriptor {
                 min_value: f32::MIN,
                 max_value: f32::MAX,
-                default_value: default_gain,
+                default_value: 0.,
                 automation_rate: crate::param::AutomationRate::A,
             };
             let (g_param, g_proc) = context
@@ -241,7 +214,6 @@ impl BiquadFilterNode {
             let (sender, receiver) = crossbeam_channel::bounded(0);
 
             let config = RendererConfig {
-                sample_rate,
                 gain: g_proc,
                 detune: d_proc,
                 frequency: f_proc,
@@ -252,7 +224,6 @@ impl BiquadFilterNode {
 
             let renderer = BiquadFilterRenderer::new(config);
             let node = Self {
-                sample_rate,
                 registration,
                 channel_config: options.channel_config.into(),
                 type_,
@@ -269,25 +240,25 @@ impl BiquadFilterNode {
 
     /// Returns the gain audio paramter
     #[must_use]
-    pub const fn gain(&self) -> &AudioParam {
+    pub fn gain(&self) -> &AudioParam {
         &self.gain
     }
 
     /// Returns the frequency audio paramter
     #[must_use]
-    pub const fn frequency(&self) -> &AudioParam {
+    pub fn frequency(&self) -> &AudioParam {
         &self.frequency
     }
 
     /// Returns the detune audio paramter
     #[must_use]
-    pub const fn detune(&self) -> &AudioParam {
+    pub fn detune(&self) -> &AudioParam {
         &self.detune
     }
 
     /// Returns the Q audio paramter
     #[must_use]
-    pub const fn q(&self) -> &AudioParam {
+    pub fn q(&self) -> &AudioParam {
         &self.q
     }
 
@@ -302,7 +273,7 @@ impl BiquadFilterNode {
     /// # Arguments
     ///
     /// * `type_` - the biquad filter type (lowpass, highpass,...)
-    pub fn set_type(&mut self, type_: BiquadFilterType) {
+    pub fn set_type(&self, type_: BiquadFilterType) {
         self.type_.store(type_ as u32, Ordering::SeqCst);
     }
 
@@ -336,7 +307,7 @@ impl BiquadFilterNode {
             Ok([b0, b1, b2, a1, a2]) => {
                 for (i, &f) in frequency_hz.iter().enumerate() {
                     let f = f64::from(f);
-                    let sample_rate = f64::from(self.sample_rate);
+                    let sample_rate = f64::from(self.context().sample_rate_raw().0);
                     let num = b0
                         + Complex::from_polar(b1, -1.0 * 2.0 * PI * f / sample_rate)
                         + Complex::from_polar(b2, -2.0 * 2.0 * PI * f / sample_rate);
@@ -375,7 +346,7 @@ impl BiquadFilterNode {
 
         // Ensures that given frequencies are in the correct range
         let min = 0.;
-        let max = self.sample_rate / 2.;
+        let max = self.context().sample_rate() / 2.;
         for f in frequency_hz.iter_mut() {
             *f = f.clamp(min, max);
         }
@@ -424,8 +395,6 @@ struct CoeffsConfig {
 /// Helper struct which regroups all parameters
 /// required to build `BiquadFilterRenderer`
 struct RendererConfig {
-    /// Sample rate (equals to audio context sample rate)
-    sample_rate: f32,
     /// quality factor - its impact on the frequency response of the filter
     /// depends on the `BiquadFilterType`
     q: AudioParamId,
@@ -461,8 +430,6 @@ struct Coefficients {
 
 /// `BiquadFilterRenderer` represents the rendering part of `BiquadFilterNode`
 struct BiquadFilterRenderer {
-    /// Sample rate (equals to audio context sample rate)
-    sample_rate: f32,
     /// quality factor - its impact on the frequency response of the filter
     /// depends on the `BiquadFilterType`
     q: AudioParamId,
@@ -494,7 +461,7 @@ impl AudioProcessor for BiquadFilterRenderer {
         outputs: &mut [AudioRenderQuantum],
         params: AudioParamValues,
         _timestamp: f64,
-        _sample_rate: SampleRate,
+        sample_rate: SampleRate,
     ) -> bool {
         // single input/output node
         let input = &inputs[0];
@@ -505,7 +472,15 @@ impl AudioProcessor for BiquadFilterRenderer {
         let freq_values = params.get(&self.frequency);
         let q_values = params.get(&self.q);
 
-        self.filter(input, output, g_values, det_values, freq_values, q_values);
+        self.filter(
+            input,
+            output,
+            g_values,
+            det_values,
+            freq_values,
+            q_values,
+            sample_rate,
+        );
 
         true // todo tail time - issue #34
     }
@@ -518,7 +493,6 @@ impl BiquadFilterRenderer {
     #[allow(clippy::missing_const_for_fn)]
     fn new(config: RendererConfig) -> Self {
         let RendererConfig {
-            sample_rate,
             q,
             detune,
             frequency,
@@ -539,7 +513,6 @@ impl BiquadFilterRenderer {
         let s2 = [0.; MAX_CHANNELS];
 
         Self {
-            sample_rate,
             gain,
             detune,
             frequency,
@@ -560,6 +533,7 @@ impl BiquadFilterRenderer {
     /// * `output` - Audiobuffer output
     /// * `params` - biquadfilter params which resolves into biquad coeffs
     #[inline]
+    #[allow(clippy::too_many_arguments)]
     fn filter(
         &mut self,
         input: &AudioRenderQuantum,
@@ -568,6 +542,7 @@ impl BiquadFilterRenderer {
         det_values: &[f32],
         freq_values: &[f32],
         q_values: &[f32],
+        sample_rate: SampleRate,
     ) {
         for (channel_idx, (i_data, o_data)) in input
             .channels()
@@ -585,7 +560,7 @@ impl BiquadFilterRenderer {
                 };
 
                 // A-rate params
-                self.update_coeffs(&p);
+                self.update_coeffs(&p, sample_rate);
                 *o = self.tick(i, channel_idx);
             }
         }
@@ -625,7 +600,7 @@ impl BiquadFilterRenderer {
     ///
     /// * `params` - params resolving into biquad coeffs
     #[inline]
-    fn update_coeffs(&mut self, params: &CoeffsConfig) {
+    fn update_coeffs(&mut self, params: &CoeffsConfig, sample_rate: SampleRate) {
         let CoeffsConfig {
             q,
             detune,
@@ -636,7 +611,7 @@ impl BiquadFilterRenderer {
 
         let computed_freq = frequency * 10_f32.powf(detune / 1200.);
 
-        let sample_rate = f64::from(self.sample_rate);
+        let sample_rate = f64::from(sample_rate.0);
         let computed_freq = f64::from(computed_freq);
         let q = f64::from(*q);
         let gain = f64::from(*gain);
@@ -1104,18 +1079,18 @@ mod test {
     use float_eq::assert_float_eq;
 
     use crate::{
-        context::{AsBaseAudioContext, OfflineAudioContext},
+        context::{BaseAudioContext, OfflineAudioContext},
         SampleRate,
     };
 
-    use super::{BiquadFilterNode, BiquadFilterOptions, BiquadFilterType, ChannelConfigOptions};
+    use super::{BiquadFilterNode, BiquadFilterOptions, BiquadFilterType};
 
     const LENGTH: usize = 555;
 
     #[test]
     fn build_with_new() {
         let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let _biquad = BiquadFilterNode::new(&context, None);
+        let _biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
     }
 
     #[test]
@@ -1125,66 +1100,17 @@ mod test {
     }
 
     #[test]
-    fn default_audio_params_are_correct_with_no_options() {
-        let default_q = 1.0;
-        let default_detune = 0.;
-        let default_gain = 0.;
-        let default_freq = 350.;
-        let default_type = BiquadFilterType::Lowpass;
-        let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let biquad = BiquadFilterNode::new(&context, None);
-
-        context.start_rendering();
-
-        assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
-        assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
-        assert_float_eq!(biquad.gain().value(), default_gain, abs <= 0.);
-        assert_float_eq!(biquad.frequency().value(), default_freq, abs <= 0.);
-        assert_eq!(biquad.type_(), default_type);
-    }
-
-    #[test]
-    fn default_audio_params_are_correct_with_no_options_in_options() {
+    fn test_default_audio_params() {
         let default_q = 1.0;
         let default_detune = 0.;
         let default_gain = 0.;
         let default_freq = 350.;
         let default_type = BiquadFilterType::Lowpass;
 
-        let options = BiquadFilterOptions {
-            q: None,
-            detune: None,
-            frequency: None,
-            gain: None,
-            type_: None,
-            channel_config: ChannelConfigOptions::default(),
-        };
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let biquad = BiquadFilterNode::new(&context, Some(options));
+        let biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
 
-        context.start_rendering();
-
-        assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
-        assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
-        assert_float_eq!(biquad.gain().value(), default_gain, abs <= 0.);
-        assert_float_eq!(biquad.frequency().value(), default_freq, abs <= 0.);
-        assert_eq!(biquad.type_(), default_type);
-    }
-
-    #[test]
-    fn default_audio_params_are_correct_with_default_options() {
-        let default_q = 1.0;
-        let default_detune = 0.;
-        let default_gain = 0.;
-        let default_freq = 350.;
-        let default_type = BiquadFilterType::Lowpass;
-        let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-
-        let options = BiquadFilterOptions::default();
-
-        let biquad = BiquadFilterNode::new(&context, Some(options));
-
-        context.start_rendering();
+        context.start_rendering_sync();
 
         assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
         assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
@@ -1203,17 +1129,17 @@ mod test {
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
         let options = BiquadFilterOptions {
-            q: Some(q),
-            detune: Some(detune),
-            gain: Some(gain),
-            frequency: Some(frequency),
-            type_: Some(type_),
+            q,
+            detune,
+            gain,
+            frequency,
+            type_,
             ..BiquadFilterOptions::default()
         };
 
-        let biquad = BiquadFilterNode::new(&context, Some(options));
+        let biquad = BiquadFilterNode::new(&context, options);
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         assert_float_eq!(biquad.q().value(), q, abs <= 0.);
         assert_float_eq!(biquad.detune().value(), detune, abs <= 0.);
@@ -1231,7 +1157,7 @@ mod test {
         let type_ = BiquadFilterType::Highpass;
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
-        let mut biquad = BiquadFilterNode::new(&context, None);
+        let biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
 
         biquad.q().set_value(q);
         biquad.detune().set_value(detune);
@@ -1239,7 +1165,7 @@ mod test {
         biquad.frequency().set_value(frequency);
         biquad.set_type(type_);
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         assert_float_eq!(biquad.q().value(), q, abs <= 0.);
         assert_float_eq!(biquad.detune().value(), detune, abs <= 0.);
@@ -1252,7 +1178,7 @@ mod test {
     #[should_panic]
     fn panics_when_not_the_same_length() {
         let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let biquad = BiquadFilterNode::new(&context, None);
+        let biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
 
         let mut frequency_hz = [0.];
         let mut mag_response = [0., 1.0];
@@ -1265,7 +1191,7 @@ mod test {
     #[should_panic]
     fn panics_when_not_the_same_length_2() {
         let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let biquad = BiquadFilterNode::new(&context, None);
+        let biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
 
         let mut frequency_hz = [0.];
         let mut mag_response = [0.];
@@ -1277,10 +1203,8 @@ mod test {
     #[test]
     fn frequencies_are_clamped() {
         let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let biquad = BiquadFilterNode::new(&context, None);
-        // It will be fine for the usual fs
-        #[allow(clippy::cast_precision_loss)]
-        let niquyst = context.sample_rate().0 as f32 / 2.0;
+        let biquad = BiquadFilterNode::new(&context, BiquadFilterOptions::default());
+        let niquyst = context.sample_rate() / 2.0;
 
         let mut frequency_hz = [-100., 1_000_000.];
         let mut mag_response = [0., 0.];
