@@ -3,7 +3,7 @@ use std::f32::consts::PI;
 use std::sync::atomic::{AtomicU32, AtomicUsize, Ordering};
 use std::sync::Arc;
 
-use crate::context::{AudioContextRegistration, AudioNodeId, BaseAudioContext};
+use crate::context::{AudioContextRegistration, AudioNodeId, ConcreteBaseAudioContext};
 use crate::control::{Controller, ScheduledState, Scheduler};
 use crate::media::MediaStream;
 use crate::render::{AudioParamValues, AudioProcessor, AudioRenderQuantum};
@@ -31,8 +31,8 @@ mod constant_source;
 pub use constant_source::*;
 mod panner;
 pub use panner::*;
-mod analyzer;
-pub use analyzer::*;
+mod analyser;
+pub use analyser::*;
 mod audio_buffer_source;
 pub use audio_buffer_source::*;
 mod media_element;
@@ -44,18 +44,15 @@ pub use waveshaper::*;
 mod stereo_panner;
 pub use stereo_panner::*;
 
-const TABLE_LENGTH_USIZE: usize = 2048;
-const TABLE_LENGTH_BY_4_USIZE: usize = TABLE_LENGTH_USIZE / 4;
-// 2048 casts without loss of precision cause its mantissa is 0b0
-#[allow(clippy::cast_precision_loss)]
-const TABLE_LENGTH_F32: f32 = TABLE_LENGTH_USIZE as f32;
-const TABLE_LENGTH_BY_4_F32: f32 = TABLE_LENGTH_BY_4_USIZE as f32;
+pub(crate) const TABLE_LENGTH_USIZE: usize = 8192;
+pub(crate) const TABLE_LENGTH_BY_4_USIZE: usize = TABLE_LENGTH_USIZE / 4;
+
+pub(crate) const TABLE_LENGTH_F32: f32 = TABLE_LENGTH_USIZE as f32;
+pub(crate) const TABLE_LENGTH_BY_4_F32: f32 = TABLE_LENGTH_BY_4_USIZE as f32;
 
 // Compute one period sine wavetable of size TABLE_LENGTH
 lazy_static! {
-    static ref SINETABLE: Vec<f32> = {
-        #[allow(clippy::cast_precision_loss)]
-        // 0 through 2048 are cast without loss of precision
+    pub(crate) static ref SINETABLE: Vec<f32> = {
         let table: Vec<f32> = (0..TABLE_LENGTH_USIZE)
             .map(|x| ((x as f32) * 2.0 * PI * (1. / (TABLE_LENGTH_F32))).sin())
             .collect();
@@ -180,7 +177,7 @@ impl From<ChannelConfigOptions> for ChannelConfig {
 /// to the audio hardware. Each node can have inputs and/or outputs.
 ///
 /// Note that the AudioNode is typically constructed together with an [`AudioProcessor`]
-/// (the object that lives the render thread). See [`BaseAudioContext::register`].
+/// (the object that lives the render thread). See [`ConcreteBaseAudioContext::register`].
 pub trait AudioNode {
     fn registration(&self) -> &AudioContextRegistration;
 
@@ -193,33 +190,46 @@ pub trait AudioNode {
     }
 
     /// The BaseAudioContext which owns this AudioNode.
-    fn context(&self) -> &BaseAudioContext {
+    fn context(&self) -> &ConcreteBaseAudioContext {
         self.registration().context()
     }
 
     /// Connect the output of this AudioNode to the input of another node.
+    ///
+    /// # Panics
+    ///
+    /// This function will panic when
+    /// - the AudioContext of the source and destination does not match
     fn connect<'a>(&self, dest: &'a dyn AudioNode) -> &'a dyn AudioNode {
-        self.connect_at(dest, 0, 0).unwrap()
+        self.connect_at(dest, 0, 0)
     }
 
     /// Connect a specific output of this AudioNode to a specific input of another node.
+    ///
+    /// # Panics
+    ///
+    /// This function will panic when
+    /// - the AudioContext of the source and destination does not match
+    /// - if the input port is out of bounds for the destination node
+    /// - if the output port is out of bounds for the source node
     fn connect_at<'a>(
         &self,
         dest: &'a dyn AudioNode,
         output: u32,
         input: u32,
-    ) -> Result<&'a dyn AudioNode, crate::IndexSizeError> {
+    ) -> &'a dyn AudioNode {
         if self.context() != dest.context() {
-            panic!("attempting to connect nodes from different contexts");
+            panic!("InvalidAccessError: Attempting to connect nodes from different contexts");
         }
-
-        if self.number_of_outputs() <= output || dest.number_of_inputs() <= input {
-            return Err(crate::IndexSizeError {});
+        if self.number_of_outputs() <= output {
+            panic!("IndexSizeError: output port {} is out of bounds", output);
+        }
+        if dest.number_of_inputs() <= input {
+            panic!("IndexSizeError: input port {} is out of bounds", input);
         }
 
         self.context().connect(self.id(), dest.id(), output, input);
-
-        Ok(dest)
+        dest
     }
 
     /// Disconnects all outputs of the AudioNode that go to a specific destination AudioNode.
