@@ -1,19 +1,17 @@
 //! The stereo panner control and renderer parts
-#![warn(
-    clippy::all,
-    clippy::pedantic,
-    clippy::nursery,
-    clippy::perf,
-    clippy::missing_docs_in_private_items
-)]
-
-use std::f32::consts::PI;
+// #![warn(
+//     clippy::all,
+//     clippy::pedantic,
+//     clippy::perf,
+//     clippy::missing_docs_in_private_items
+// )]
 
 use float_eq::debug_assert_float_eq;
+use std::f32::consts::PI;
 
 use crate::{
-    context::{AsBaseAudioContext, AudioContextRegistration, AudioParamId},
-    param::{AudioParam, AudioParamOptions},
+    context::{AudioContextRegistration, AudioParamId, BaseAudioContext},
+    param::{AudioParam, AudioParamDescriptor},
     render::{AudioParamValues, AudioProcessor, AudioRenderQuantum},
     SampleRate,
 };
@@ -23,14 +21,14 @@ use super::{
     SINETABLE, TABLE_LENGTH_BY_4_F32, TABLE_LENGTH_BY_4_USIZE,
 };
 
-/// `StereoPannerOptions` is used to pass options
-/// during the construction of `StereoPannerNode` using its
-/// constructor method `new`
-// the naming comes from the web audio specfication
-#[allow(clippy::module_name_repetitions)]
+/// Options for constructing a [`StereoPannerOptions`]
+// dictionary StereoPannerOptions : AudioNodeOptions {
+//   float pan = 0;
+// };
+#[derive(Clone, Debug)]
 pub struct StereoPannerOptions {
     /// initial value for the pan parameter
-    pan: Option<f32>,
+    pub pan: f32,
     /// audio node options
     pub channel_config: ChannelConfigOptions,
 }
@@ -38,7 +36,7 @@ pub struct StereoPannerOptions {
 impl Default for StereoPannerOptions {
     fn default() -> Self {
         Self {
-            pan: Some(0.),
+            pan: 0.,
             channel_config: ChannelConfigOptions {
                 count: 2,
                 mode: ChannelCountMode::ClampedMax,
@@ -49,8 +47,6 @@ impl Default for StereoPannerOptions {
 }
 
 /// `StereoPannerNode` positions an incoming audio stream in a stereo image
-// the naming comes from the web audio specfication
-#[allow(clippy::module_name_repetitions)]
 pub struct StereoPannerNode {
     /// Represents the node instance and its associated audio context
     registration: AudioContextRegistration,
@@ -106,10 +102,8 @@ impl StereoPannerNode {
     ///
     /// * `context` - audio context in which the audio node will live.
     /// * `options` - stereo panner options
-    pub fn new<C: AsBaseAudioContext>(context: &C, options: Option<StereoPannerOptions>) -> Self {
+    pub fn new<C: BaseAudioContext>(context: &C, options: StereoPannerOptions) -> Self {
         context.base().register(move |registration| {
-            let options = options.unwrap_or_default();
-
             assert!(
                 options.channel_config.count <= 2,
                 "NotSupportedError: channel count"
@@ -119,14 +113,12 @@ impl StereoPannerNode {
                 "NotSupportedError: count mode"
             );
 
-            let default_pan = 0.;
+            let pan_value = options.pan;
 
-            let pan_value = options.pan.unwrap_or(default_pan);
-
-            let pan_param_opts = AudioParamOptions {
+            let pan_param_opts = AudioParamDescriptor {
                 min_value: -1.,
                 max_value: 1.,
-                default_value: default_pan,
+                default_value: 0.,
                 automation_rate: crate::param::AutomationRate::A,
             };
             let (pan_param, pan_proc) = context
@@ -148,7 +140,7 @@ impl StereoPannerNode {
 
     /// Returns the pan audio paramter
     #[must_use]
-    pub const fn pan(&self) -> &AudioParam {
+    pub fn pan(&self) -> &AudioParam {
         &self.pan
     }
 }
@@ -268,17 +260,17 @@ mod test {
     use float_eq::assert_float_eq;
 
     use crate::{
-        context::{AsBaseAudioContext, OfflineAudioContext},
+        context::{BaseAudioContext, OfflineAudioContext},
         SampleRate,
     };
 
-    use super::{StereoPannerNode, StereoPannerRenderer};
+    use super::{StereoPannerNode, StereoPannerOptions, StereoPannerRenderer};
     const LENGTH: usize = 555;
 
     #[test]
     fn build_with_new() {
         let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
-        let _panner = StereoPannerNode::new(&context, None);
+        let _panner = StereoPannerNode::new(&context, StereoPannerOptions::default());
     }
 
     #[test]
@@ -293,9 +285,9 @@ mod test {
 
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
-        let panner = StereoPannerNode::new(&context, None);
+        let panner = StereoPannerNode::new(&context, StereoPannerOptions::default());
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, default_pan, abs_all <= 0.);
@@ -308,13 +300,13 @@ mod test {
 
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
-        let panner = StereoPannerNode::new(&context, None);
+        let panner = StereoPannerNode::new(&context, StereoPannerOptions::default());
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, default_pan, abs_all <= 0.);
         panner.pan().set_value(new_pan);
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, new_pan, abs_all <= 0.);
@@ -365,13 +357,13 @@ mod test {
 
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
-        let panner = StereoPannerNode::new(&context, None);
+        let panner = StereoPannerNode::new(&context, StereoPannerOptions::default());
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, default_pan, abs_all <= 0.);
         panner.pan().set_value(new_pan);
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, new_pan, abs_all <= 0.);
@@ -385,13 +377,13 @@ mod test {
 
         let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
-        let panner = StereoPannerNode::new(&context, None);
+        let panner = StereoPannerNode::new(&context, StereoPannerOptions::default());
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, default_pan, abs_all <= 0.);
         panner.pan().set_value(new_pan);
 
-        context.start_rendering();
+        context.start_rendering_sync();
 
         let pan = panner.pan.value();
         assert_float_eq!(pan, new_pan, abs_all <= 0.);
