@@ -62,17 +62,14 @@ impl Node {
             return false;
         }
 
-        // Drop, if the node has no outputs connected
-        if self.outgoing_edges.is_empty() {
-            return true;
-        }
-
         // Drop, when the node does not have any inputs connected,
         // and if the processor reports it won't yield output.
         if !self.has_inputs_connected && !tail_time {
             return true;
         }
 
+        // Otherwise, do not drop the node.
+        // (Even if it has no outputs connected, it may have side effects)
         false
     }
 
@@ -173,7 +170,12 @@ impl Graph {
     }
 
     pub fn mark_free_when_finished(&mut self, index: NodeIndex) {
-        self.nodes.get_mut(&index).unwrap().free_when_finished = true;
+        // Issue #92, a race condition can occur for AudioParams. They may have already been
+        // removed from the audio graph if the node they feed into was dropped.
+        // Therefore, do not assume this node still exists:
+        if let Some(node) = self.nodes.get_mut(&index) {
+            node.free_when_finished = true;
+        }
     }
 
     /// Helper function for `order_nodes` - traverse node and outgoing edges
@@ -225,7 +227,7 @@ impl Graph {
     /// node, at every render quantum the source should render first and after that the gain node.
     ///
     /// Inspired by the spec recommendation at
-    /// https://webaudio.github.io/web-audio-api/#rendering-loop
+    /// <https://webaudio.github.io/web-audio-api/#rendering-loop>
     ///
     /// The goals are:
     /// - Perform a topological sort of the graph
@@ -247,7 +249,7 @@ impl Graph {
 
         // Visit all registered nodes, and perform a depth first traversal.
         //
-        // We cannot just start from the DestinationNode and visit all nodes connecting to it,
+        // We cannot just start from the AudioDestinationNode and visit all nodes connecting to it,
         // since the audio graph could contain legs detached from the destination and those should
         // still be rendered.
         self.nodes.keys().for_each(|&node_id| {
@@ -327,15 +329,15 @@ impl Graph {
 
             // Check if we can decommission this node (end of life)
             if node.can_free(tail_time) {
-                // Other nodes could still connect to this one, clear these edges
-                nodes
-                    .values_mut()
-                    .for_each(|n| n.outgoing_edges.retain(|e| e.other_id != *index));
-
-                // We should perform a new topological sort of the graph now
+                // Node is dropped, we should perform a new topological sort of the audio graph
                 nodes_dropped = true;
+
+                // Nodes are only dropped when they do not have incoming connections.
+                // But they may have AudioParams feeding into them, these can de dropped too.
+                nodes.retain(|_id, n| !n.outgoing_edges.iter().any(|e| e.other_id == *index));
             } else {
-                // Reset input buffers (we don't need them so this saves allocations)
+                // Node is not dropped.
+                // Reset input buffers as they will be summed up in the next render quantum.
                 node.inputs
                     .iter_mut()
                     .for_each(AudioRenderQuantum::make_silent);
