@@ -1,18 +1,18 @@
 //! Spatialization/Panning primitives
 //!
 //! Required for panning algorithm, distance and cone effects of [`crate::node::PannerNode`]s
-use crate::context::{AsBaseAudioContext, AudioContextRegistration, AudioParamId};
+use crate::context::{AudioContextRegistration, AudioParamId, BaseAudioContext};
 use crate::node::{
     AudioNode, ChannelConfig, ChannelConfigOptions, ChannelCountMode, ChannelInterpretation,
 };
-use crate::param::{AudioParam, AudioParamOptions, AudioParamRaw, AutomationRate};
+use crate::param::{AudioParam, AudioParamDescriptor, AudioParamRaw, AutomationRate};
 use crate::render::{AudioParamValues, AudioProcessor, AudioRenderQuantum};
 use crate::SampleRate;
 
 use std::f32::consts::PI;
 
 /// AudioParam settings for the carthesian coordinates
-pub(crate) const PARAM_OPTS: AudioParamOptions = AudioParamOptions {
+pub(crate) const PARAM_OPTS: AudioParamDescriptor = AudioParamDescriptor {
     min_value: f32::MIN,
     max_value: f32::MAX,
     default_value: 0.,
@@ -69,8 +69,8 @@ impl AudioListener {
 /// This node has no input, but takes the position/orientation AudioParams and copies them into the
 /// 9 outputs. The outputs are connected to the PannerNodes (via an AudioParam).
 ///
-/// The AudioListener is always connected to the DestinationNode so each render quantum its
-/// positions are recalculated.
+/// The AudioListener is always connected to the AudioDestinationNode so at each
+/// render quantum its positions are recalculated.
 pub(crate) struct AudioListenerNode {
     registration: AudioContextRegistration,
     fields: AudioListener,
@@ -116,16 +116,16 @@ impl AudioNode for AudioListenerNode {
 }
 
 impl AudioListenerNode {
-    pub fn new<C: AsBaseAudioContext>(context: &C) -> Self {
+    pub fn new<C: BaseAudioContext>(context: &C) -> Self {
         context.base().register(move |registration| {
             let reg_id = registration.id();
             let base = context.base();
 
-            let forward_z_opts = AudioParamOptions {
+            let forward_z_opts = AudioParamDescriptor {
                 default_value: -1.,
                 ..PARAM_OPTS
             };
-            let up_y_opts = AudioParamOptions {
+            let up_y_opts = AudioParamDescriptor {
                 default_value: 1.,
                 ..PARAM_OPTS
             };
